/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: CursesModel.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.cc;

import java.awt.Paint;

/**
 * A data model for a <code>Curses</code> component.
 *
 * @author Steve Millington
 * @version $Id: CursesModel.java 256 2008-07-15 15:54:02Z steve $
 */
public interface CursesModel {

    /**
     * Add a listener to the model.
     *
     * @param l The listener
     */
    void addModelListener(CursesModelListener l);

    /**
     * Remove a listener from the model.
     *
     * @param l The listener
     */
    void removeModelListener(CursesModelListener l);

    /**
     * Get the cursor location horizontally.
     *
     * @return The X position
     */
    int getCursorX();

    /**
     * Get the cursor location vertically.
     *
     * @return The Y position
     */
    int getCursorY();

    /**
     * Determine the number of rows of data in the model.
     *
     * @return The row count
     */
    int getRows();

    /**
     * Determine the number of columns of data in the model.
     *
     * @return The column count
     */
    int getCols();

    /**
     * Get the character at the given location.
     *
     * @param x The x location
     * @param y The y location
     * @return The character
     */
    char getCharAt(int x, int y);

    /**
     * Get the FG Color at the given location.
     *
     * @param x The x location
     * @param y The y location
     * @return The attribute
     */
    Paint getForegroundAt(int x, int y);

    /**
     * Get the BG Color at the given location.
     *
     * @param x The x location
     * @param y The y location
     * @return The attribute
     */
    Paint getBackgroundAt(int x, int y);

    /**
     * Get the underline status of the given location.
     *
     * @param x The x location
     * @param y The y location
     * @return True if the character is underlined
     */
    boolean isUnderLine(int x, int y);

    /**
     * Is the cursor supposed to be visible.
     *
     * @return true for visible.
     */
    boolean isCursorVisible();
}
