/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: PackageManager.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.cc;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;

/**
 * A class managing the UI of this package.
 *
 * @author Steve Millington
 * @version $Id: PackageManager.java 256 2008-07-15 15:54:02Z steve $
 */
public final class PackageManager {

    private static final String THIS_PACKAGE = "uk.co.ruelgnoj.orion.cc.";
    private static final String CC_UICLASSID = "CursesUI";

    private static final ThreadLocal<Object> initialized = new ThreadLocal<>();

    /**
     * Private constructor to prevent construction.
     */
    private PackageManager() {
    }

    /**
     * Initialize this package. This method registers the UI things with
     * appropriate people.
     */
    public static void initialize() {
	//
        // BEWARE:
        // This uses thread local storage to deal with initialization. This
        // is not necessary for applications, but in applets it is. If a
        // normal piece of static is used, then pressing 'reload' in
        // a browser results in the UI class not being found.
        // This seems to have something to do with the Swing thread changing
        // underneath us!
        if (initialized.get() == null) {
            setLookAndFeelProperties(UIManager.getLookAndFeel());

            UIManager.addPropertyChangeListener(
                    new PropertyChangeListener() {
                        @Override
                        public void propertyChange(PropertyChangeEvent e) {
                            if ("lookAndFeel".equals(e.getPropertyName())) {
                                setLookAndFeelProperties(
                                        (LookAndFeel) e.getNewValue());
                            }
                        }
                    });
            initialized.set(new Object());
        }
    }

    /**
     *
     * Set the properties of the LAF.
     *
     * @param laf The LAF
     */
    private static void setLookAndFeelProperties(LookAndFeel laf) {
        if (laf == null) {
            return;
        }
        switch (laf.getID()) {
            case "Metal":
                UIManager.put(CC_UICLASSID, THIS_PACKAGE + "basic.BasicCursesUI");
                break;
            case "Windows":
                UIManager.put(CC_UICLASSID, THIS_PACKAGE + "basic.BasicCursesUI");
                break;
            default:
                UIManager.put(CC_UICLASSID, THIS_PACKAGE + "basic.BasicCursesUI");
                break;
        }
    }
}
