/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: AbstractView.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import javax.swing.ActionMap;
import javax.swing.InputMap;

/**
 * A base view that provides for management of active state, and reference to
 * a Orion component.
 * @author Steve Millington
 * @version $Id: AbstractView.java 256 2008-07-15 15:54:02Z steve $
 */
public abstract class AbstractView implements View {
    /** The component for this view. */
    protected final OrionComponent	orionComponent;

    private boolean		active;
    private final ActionMap actionMap;
    private final InputMap inputMap;

    /**
     * Constructor supplying an owning component.
     * @param orionComponent The owning component.
     */
    protected AbstractView(OrionComponent orionComponent) {
    	this.orionComponent = orionComponent;
    	active = false;
	actionMap = new ActionMap();
	inputMap = new InputMap();
	initializeActionMap(actionMap);
	initializeInputMap(inputMap);
    }

    @Override
    public void activate() {
    	active = true;
    }

    @Override
    public void deactivate() {
    	active = false;
    }

    @Override
    public boolean isActive() {
    	return active;
    }

    @Override
    public final InputMap getInputMap() {
	return inputMap;
    }

    @Override
    public final ActionMap getActionMap() {
	return actionMap;
    }

    /**
     * Initialized the action map. By default this does nothing, and subclasses
     * are expected to fill actions in.
     * @param am The Action Map.
     */
    protected void initializeActionMap(ActionMap am) {
	// Default is to do nothing.
    }

    /**
     * Initialized the input map. By default this does nothing, and subclasses
     * are expected to fill actions in.
     * @param im The Input Map.
     */
    protected void initializeInputMap(InputMap im) {
	// Default is to do nothing.
    }
}
