/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: ExceptionView.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import java.awt.Color;

import java.io.PrintWriter;
import java.io.StringWriter;

import uk.co.ruelgnoj.orion.cc.WritableCursesModel;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;

/**
 * View an exception.
 *
 * @author Steve Millington
 * @version $Id: ExceptionView.java 256 2008-07-15 15:54:02Z steve $
 */
public class ExceptionView extends PlainTextView {

    private static final Color BG = Color.RED.darker();
    private static final String MSG = "EXCEPTION OCCURED";

    /**
     * Constructor.
     * @param orionComponent The parent component
     * @param t The exception to display.
     */
    public ExceptionView(OrionComponent orionComponent, Throwable t) {
	super(orionComponent);
        if (t instanceof OrionDataModelException && t.getCause() != null) {
            t = t.getCause();
        }
	final StringWriter sw = new StringWriter();
	t.printStackTrace(new PrintWriter(sw));
	setText(sw.toString());
        t.printStackTrace();
    }

    /**
     * Draw the Orion name and version information.
     */
    @Override
    protected void setTitles() {
	final WritableCursesModel headerArea = orionComponent.getHeaderArea();
    	headerArea.setBatch(true);
	headerArea.setBG(BG);
	headerArea.clear();

	final int cols = headerArea.getCols();
	final int len = MSG.length();
	headerArea.moveTo(0, 0);
	headerArea.setBG(Color.YELLOW);
	headerArea.setFG(Color.BLACK);
	headerArea.moveTo((cols - len) / 2, 1);
	headerArea.print(MSG);
	headerArea.setBG(Color.YELLOW);
	headerArea.setFG(Color.BLACK);
    	headerArea.setBatch(false);
    }

    @Override
    public void activate() {
        final WritableCursesModel mainArea = orionComponent.getMainArea();
	mainArea.setBG(BG);
	mainArea.setFG(Color.YELLOW);
	super.activate();
    }
}
