/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: FileColorPanel.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.component;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.BorderFactory;
import javax.swing.text.StyleContext;
import javax.swing.text.StyleConstants;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;

import uk.co.ruelgnoj.orion.tokenizers.LanguageTokenizer;
import uk.co.ruelgnoj.orion.tokenizers.TokenType;

public class FileColorPanel extends JPanel {

    private final FileViewSettings fvs;
    private final ColorChooser[] choosers;

    public FileColorPanel(FileViewSettings fvs) {
        super(new BorderLayout());
        this.fvs = fvs.clone();
        final JPanel p = new JPanel(new GridLayout(0, 1));
        final LanguageTokenizer tokenizer = fvs.getTokenizer();
        final Set<TokenType> tokenTypes = getSortedTokenTypes(tokenizer);
        choosers = new ColorChooser[tokenTypes.size()];
        final StyleContext sc = fvs.getStyleContext();
        int ci = 0;
        for (TokenType tt : tokenTypes) {
            choosers[ci++] = new ColorChooser(
                    tt.getDisplayName(),
                    StyleConstants.getForeground(
                            sc.getStyle(tt.getName())));
        }
        for (int i = 0; i < ci; i++) {
            p.add(choosers[i]);
        }
        add(BorderLayout.NORTH, p);
    }

    //
    // Should this go into the Tokenizer class?
    //
    private static Set<TokenType> getSortedTokenTypes(LanguageTokenizer tokenizer) {
        final Set<TokenType> tokenTypes = tokenizer.getPossibleTokenTypes();
        final Set<TokenType> tmp = new TreeSet<>(
                new Comparator<TokenType>() {
                    @Override
                    public int compare(TokenType t1, TokenType t2) {
                        return t1.getDisplayName().compareTo(t2.getDisplayName());
                    }
                });
        tmp.addAll(tokenTypes);
        return tmp;
    }

    public FileViewSettings getFVS() {
        setValues();
        return fvs;
    }

    private static class ColorChooser extends JPanel
            implements ActionListener {

        private static final Dimension EXAMPLE_SIZE = new Dimension(20, 20);
        private static final Insets EMPTY = new Insets(0, 0, 0, 0);
        private final JComponent example;
        private final JLabel l;

        public ColorChooser(String name, Color clr) {
            setLayout(new BorderLayout());
            final JButton b = new JButton("...");
            b.setMargin(EMPTY);
            b.addActionListener(this);
            final Container p = new JPanel();
            p.setLayout(new FlowLayout());
            l = new JLabel(name, JLabel.RIGHT);
            add(BorderLayout.CENTER, l);
            example = new JPanel();
            example.setPreferredSize(EXAMPLE_SIZE);
            example.setBackground(clr);
            example.setBorder(
                    BorderFactory.createCompoundBorder(
                            BorderFactory.createLineBorder(Color.black),
                            BorderFactory.createLineBorder(Color.white)));
            p.add(example);
            p.add(b);
            add(BorderLayout.EAST, p);
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            final Color c = JColorChooser.showDialog(this,
                    "Hello",
                    example.getBackground());
            if (c != null) {
                example.setBackground(c);
            }
        }

        public Color getColor() {
            return example.getBackground();
        }
    }

    private void setValues() {
        /*
         fvs.backgroundColor = choosers[0].getColor();
         fvs.whitespaceColor = choosers[1].getColor();
         fvs.commentColor = choosers[2].getColor();
         fvs.punctuationColor = choosers[3].getColor();
         fvs.stringColor = choosers[4].getColor();
         fvs.wordColor = choosers[5].getColor();
         fvs.keywordColor = choosers[6].getColor();
         fvs.numberColor = choosers[7].getColor();
         fvs.characterColor = choosers[8].getColor();
         fvs.linenumberColor = choosers[9].getColor();
         */
    }
}
