/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: Authenticator.java 225 2008-03-24 18:39:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import java.util.List;
import java.util.concurrent.ExecutionException;
import javax.swing.SwingWorker;
import uk.co.ruelgnoj.orion.dm.OrionFileEntry;

/**
 *
 * @author Steve Millington
 * @version $Id$
 */
public class HuntSpecificSwingWorker extends SwingWorker<List<OrionFileEntry>, Void> {

    private final OrionComponent oc;
    private final String searchString;

    public HuntSpecificSwingWorker(OrionComponent oc, String searchString) {
        this.oc = oc;
        this.searchString = searchString;
        doPreGUIWork();
    }

    protected void doPreGUIWork() {
        oc.setLocked(true);
        oc.getMessenger().info("Hunting specific for " + searchString + " ...");
    }

    @Override
    protected List<OrionFileEntry> doInBackground() throws Exception {
        final List<OrionFileEntry> list = oc.getModel().getFiles(searchString);
        return list;
    }

    @Override
    protected void done() {
        oc.setLocked(false);
        try {
            final List<OrionFileEntry> list = get();
            final View view = new FileListView(oc,
                    searchString,
                    list);
            oc.getMessenger().info("");
            oc.setView(view);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            oc.getMessenger().error(e.getCause().getMessage());
        }
    }
}
