/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: Authenticator.java 225 2008-03-24 18:39:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import java.util.List;
import java.util.concurrent.ExecutionException;
import javax.swing.SwingWorker;

/**
 *
 * @author Steve Millington
 * @version $Id$
 */
public class HuntSwingWorker extends SwingWorker<List<String>, Void> {

    private final OrionComponent oc;
    private final String searchString;

    public HuntSwingWorker(OrionComponent oc, String searchString) {
        this.oc = oc;
        this.searchString = searchString;
        doPreGUIWork();
    }

    protected void doPreGUIWork() {
        oc.setLocked(true);
        oc.getMessenger().info("Hunting for " + searchString + " ...");
    }

    @Override
    protected List<String> doInBackground() throws Exception {
        final List<String> list = oc.getModel().getTags(searchString);
        return list;
    }

    @Override
    protected void done() {
        oc.setLocked(false);
        try {
            final List<String> list = get();
            if ((list == null) || list.isEmpty()) {
                oc.getMessenger().error("No matches found for " + searchString);
                return;
            }
            if (list.size() == 1) {
                final String newSearchString = list.get(0);
                final HuntSpecificSwingWorker hsw = new HuntSpecificSwingWorker(oc, newSearchString);
                hsw.execute();
            } else {
                final View view = new VariableListView(oc,
                        searchString,
                        list);
                oc.setView(view);
            }

        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            oc.getMessenger().error(e.getCause().getMessage());
        }
    }
}
