/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: Authenticator.java 225 2008-03-24 18:39:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import java.util.concurrent.ExecutionException;
import javax.swing.SwingWorker;
import uk.co.ruelgnoj.orion.component.fileview.FileView;

/**
 *
 * @author Steve Millington
 * @version $Id$
 */
public class OpenFileSwingWorker extends SwingWorker<CharSequence, Void> {

    private final OrionComponent oc;
    private final String fileName;
    private final String searchString;

    public OpenFileSwingWorker(OrionComponent oc, String fileName, String searchString) {
        this.oc = oc;
        this.fileName = fileName;
        this.searchString = searchString;
        doPreGUIWork();
    }

    protected void doPreGUIWork() {
        oc.setLocked(true);
        oc.getMessenger().info("Opening " + fileName + " ...");
    }

    @Override
    protected CharSequence doInBackground() throws Exception {
        final CharSequence dataBuffer = oc.getModel().getFile(
                fileName);
        return dataBuffer;
    }

    @Override
    protected void done() {
        oc.setLocked(false);
        try {
            final CharSequence list = get();
            final View v = new FileView(oc,
                    fileName,
                    searchString,
                    list);
            oc.setView(v);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            final View view = new ExceptionView(oc, e.getCause());
            oc.setView(view);
        }
    }
}
