/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: PartialBorderLayout.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;


/**
 * A truncated, optimized version of border layout.
 *
 * <p><b>Note: </b>This is not a full implementation.
 * You should NOT set any east or west components
 *
 */
public class PartialBorderLayout implements LayoutManager {
    public static final String NORTH = BorderLayout.NORTH;
    public static final String SOUTH = BorderLayout.SOUTH;
    public static final String EAST = BorderLayout.EAST;
    public static final String WEST = BorderLayout.WEST;
    public static final String CENTER = BorderLayout.CENTER;

    private Component north;
    private Component south;
    private Component east;
    private Component west;
    private Component center;

    /**
     * Contstructor
     */
    public PartialBorderLayout() {
        north = null;
        south = null;
        east = null;
        west = null;
        center = null;
    }

    /** @see java.awt.LayoutManager */
    @Override
    public void addLayoutComponent(String name, Component c) {
        if (name.equals(NORTH)) {
            north = c;
        } else if (name.equals(SOUTH)) {
            south = c;
        } else if (name.equals(EAST)) {
            east = c;
        } else if (name.equals(WEST)) {
            west = c;
        } else if (name.equals(CENTER)) {
            center = c;
        }
    }

    /** @see java.awt.LayoutManager */
    @Override
    public void layoutContainer(Container parent) {
        Dimension d = parent.getSize();
        Insets insets = parent.getInsets();
        d.width -= (insets.left + insets.right);
        d.height -= (insets.top + insets.bottom);
        int hLeft = d.height;
        int nh = 0;
        int sh = 0;
        if (north != null) {
            nh = north.getPreferredSize().height;
            north.setBounds(insets.left, insets.top, d.width, nh);
            hLeft -= nh;
        }
        if (south != null) {
            sh = south.getPreferredSize().height;
            south.setBounds(insets.left, d.height - sh, d.width, sh);
            hLeft -= sh;
        }
        if (center != null) {
            center.setBounds(insets.left, insets.top + nh, d.width, hLeft);
        }
    }

    /** @see java.awt.LayoutManager */
    @Override
    public Dimension minimumLayoutSize(Container parent) {
        return new Dimension(0, 0);
    }

    /** @see java.awt.LayoutManager */
    @Override
    public Dimension preferredLayoutSize(Container parent) {
        Dimension d = new Dimension();
        Dimension nps = null;
        Dimension sps = null;
        Dimension eps = null;
        Dimension wps = null;
        Dimension cps = null;
        if (north != null) {
            nps = north.getPreferredSize();
        }
        if (south != null) {
            sps = south.getPreferredSize();
        }
        if (east != null) {
            eps = east.getPreferredSize();
        }
        if (west != null) {
            wps = west.getPreferredSize();
        }
        if (center != null) {
            cps = center.getPreferredSize();
        }


        if (north != null) {
            d.width = Math.max(d.width, nps.width);
        }
        if (south != null) {
            d.width = Math.max(d.width, sps.width);
        }

        int t;

        t = 0;
        if (east != null) {
            t += eps.width;
        }
        if (center != null) {
            t += cps.width;
        }
        if (west != null) {
            t += wps.width;
        }
        d.width = Math.max(d.width, t);

        if (north != null) {
            d.height += nps.height;
        }
        if (south != null) {
            d.height += sps.height;
        }

        t = 0;
        if (east != null) {
            t += eps.height;
        }
        if (center != null) {
            t += cps.height;
        }
        if (west != null) {
            t += wps.height;
        }
        d.height += t;

        Insets insets = parent.getInsets();
        d.width += (insets.left + insets.right);
        d.height += (insets.top + insets.bottom);

        return d;
    }

    /** @see java.awt.LayoutManager */
    @Override
    public void removeLayoutComponent(Component c) {
        if (north == c) {
            north = null;
        }
        if (south == c) {
            south = null;
        }
        if (east == c) {
            east = null;
        }
        if (west == c) {
            west = null;
        }
        if (center == c) {
            center = null;
        }
    }
}
