/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: PlainTextView.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import java.util.ArrayList;
import java.util.List;
import uk.co.ruelgnoj.orion.cc.CursesModel;
import uk.co.ruelgnoj.orion.cc.WritableCursesModel;

/**
 * A view for displaying some sort of plain text.
 * @author Steve Millington
 * @version $Id: PlainTextView.java 256 2008-07-15 15:54:02Z steve $
 */
public abstract class PlainTextView extends AbstractView {

    private final List<String> lines;
    private int topIndex;

    /**
     * Constructor.
     * @param orionComponent The component to work on.
     */
    public PlainTextView(OrionComponent orionComponent) {
        super(orionComponent);
        lines = new ArrayList<>();
    }

    /**
     * Display the text.
     */
    private void display() {
        final WritableCursesModel mainArea = orionComponent.getMainArea();
        mainArea.setBatch(true);

        mainArea.clear();
        for (int i = 0; i < mainArea.getRows(); i++) {
            if ((i + topIndex) >= lines.size()) {
                break;
            }
            mainArea.moveTo(0, i);
            mainArea.print(truncate(lines.get(topIndex + i)));
        }

        mainArea.setBatch(false);
    }
    
    private String truncate(String s) {
        final CursesModel mainArea = orionComponent.getMainArea();
        if (s.length() >  mainArea.getCols()) {
            s = s.substring(0, mainArea.getCols());
        }
        return s;
    }

    /**
     * Draw th banner for the page.
     */
    protected abstract void setTitles();

    /** {@inheritDoc} */
    @Override
    public void activate() {
        super.activate();

        setTitles();

        display();
    }

    /**
     * Set the text to view.
     * @param s The text
     */
    public void setText(String s) {
        lines.clear();
        final StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            final char c = s.charAt(i);
            if (c == '\n') {
                lines.add(sb.toString());
                sb.setLength(0);
            } else {
                sb.append(c);
            }
        }
        if (sb.length() > 0) {
            lines.add(sb.toString());
        }
    }
}
