/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: View.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component;

import javax.swing.ActionMap;
import javax.swing.InputMap;
import uk.co.ruelgnoj.orion.component.util.Activatable;

/**
 * This interface represents a view of some data as displayed in Orion.
 * <p>The Orion is essentially a stack of view implementations. Each view
 * is responsible for its own rendering etc. Although the <code>View</code>
 * interface does not have access to the base <code>OrionComponent</code>, it
 * works closely with one. Access to this base component is normally provided
 * through the <code>View</code>s constructor.
 * <p>The <code>View</code> interface provides all the information required
 * by the <code>OrionComponent</code> in order for it to manage which view is
 * the 'current' one. Only one view is 'active' at any one point, and the
 * activation/deactivation process is preformed by the
 * <code>OrionComponent</code>.
 * <p>In addition to activation, this interface also allows a view to provide
 * a set of actions ank key mappings that to be made available when the view is
 * made active.
 *
 * @author Steve Millington
 * @version $Id: View.java 256 2008-07-15 15:54:02Z steve $
 */
public interface View extends Activatable {
    /**
     * Get the input map for this view.
     * <p>This should never be <code>null</code>.
     * @return The input map.
     */
    InputMap getInputMap();

    /**
     * Get the action map for this view.
     * <p>This should never be <code>null</code>.
     * @return The action map.
     */
    ActionMap getActionMap();
}
