/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: DebugView.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component.debug;

import java.awt.Color;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import uk.co.ruelgnoj.orion.cc.WritableCursesModel;

import uk.co.ruelgnoj.orion.component.AbstractView;
import uk.co.ruelgnoj.orion.component.OrionComponent;

/**
 * A view to display debug information.
 * @author Steve Millington
 * @version $Id: DebugView.java 256 2008-07-15 15:54:02Z steve $
 */
class DebugView extends AbstractView {
    private List<String> l1;
    private List<String> l2;

    /**
     * Constructor.
     * @param orionComponent The component to operate on
     */
    public DebugView(OrionComponent orionComponent) {
	super(orionComponent);

    }

    /**
     * Render a heading.
     * @param target The place to render to
     * @param heading the heading
     */
    private static void h1(WritableCursesModel target, String heading) {
	target.setUnderLine(true);
	target.println(heading);
	target.setUnderLine(false);
    }

    /**
     * Initialize an unordered list.
     */
    private void startUl() {
	l1 = new ArrayList<>();
	l2 = new ArrayList<>();
    }

    /**
     * Add a list entry.
     * @param s1 The left item
     * @param s2 The right item
     */
    private void li(String s1, Object s2) {
	l1.add(s1);
	l2.add(s2 == null ? "null" : s2.toString());
    }

    /**
     * Add a list entry.
     * @param s1 The left item
     * @param i1 The right item
     */
    private void li(String s1, int i1) {
	li(s1, Integer.toString(i1));
    }

    /**
     * End the list. Also write it to the model.
     * @param target the model to write to
     */
    private void endUl(WritableCursesModel target) {
	int n = 0;
	for (String s : l1) {
	    n = Math.max(n, s.length());
	}
	final Iterator it1 = l1.iterator();
	final Iterator it2 = l2.iterator();
	while (it1.hasNext()) {
	    target.print("  ");
	    final String s1 = (String)it1.next();
	    target.print(s1);
	    for (int i = s1.length(); i <= n; i++) {
		target.print(' ');
	    }
	    target.print("= ");
	    final Object o = it2.next();
	    target.setFG(Color.cyan);
	    target.println(o.toString());
	    target.setFG(Color.white);
	}
    }

    /**
     * Display everything.
     */
    private void display() {
        final WritableCursesModel mainArea = orionComponent.getMainArea();
	final WritableCursesModel headerArea = orionComponent.getHeaderArea();
        final Color bgColor = Color.red.darker();
        final Color fgColor = Color.white;
        mainArea.setBatch(true);

        mainArea.setBG(bgColor);
        mainArea.setFG(fgColor);
        mainArea.clear();
	mainArea.moveTo(0, 0);

	h1(mainArea, "Header Area");
	startUl();
	li("Rows", headerArea.getRows());
	li("Cols", headerArea.getCols());
	endUl(mainArea);

	h1(mainArea, "Main Area");
	startUl();
	li("Rows", mainArea.getRows());
	li("Cols", mainArea.getCols());
	endUl(mainArea);

	/*
	h1(mainArea, "Graphics");
	FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(
							mainArea.getFont());
	int charW = fm.charWidth('W');
	int charH = fm.getHeight();
	startUl();
	li("Font", mainArea.getFont());
	li("Char Width", charW);
	li("Char Height", charH);
	endUl(mainArea);
	*/

	h1(mainArea, "Component");
	startUl();
	li("Width(pixels)", orionComponent.getWidth());
	li("Height(pixels)", orionComponent.getHeight());
	li("Locale", orionComponent.getLocale());
	endUl(mainArea);

	h1(mainArea, "Properties");
	startUl();
	li("java.version", System.getProperty("java.version"));
	li("java.vendor", System.getProperty("java.vendor"));
	endUl(mainArea);

        mainArea.setBatch(false);
    }

    /**
     * Draw the Orion name and version information.
     */
    private void setTitles() {
	final WritableCursesModel headerArea = orionComponent.getHeaderArea();
    	headerArea.setBatch(true);
	headerArea.setBG(Color.blue.darker());
	headerArea.clear();

	headerArea.moveTo(0, 0);
	headerArea.setBG(Color.yellow);
	headerArea.setFG(Color.black);
	headerArea.print("DEBUG SCREEN");
	headerArea.setBG(Color.blue.darker());

    	headerArea.setBatch(false);
    }

    @Override
    public void activate() {
	super.activate();

	setTitles();

	display();
    }
}
