/*
orion: Provides a set of source code browsing utilities.
Copyright (C) 2008  Steve Millington

$Id: CharacterLeftAction.java 285 2011-07-19 12:37:42Z steve $

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.component.fileview;

import java.awt.event.ActionEvent;

import java.util.List;

import javax.swing.AbstractAction;

import uk.co.ruelgnoj.orion.tokenizers.LanguageToken;

/**
 * File View Action - Move left 1 character.
 *
 * @author Steve Millington
 * @version $Id: CharacterLeftAction.java 285 2011-07-19 12:37:42Z steve $
 */
public class CharacterLeftAction extends AbstractAction {

    private final Internals internals;

    /**
     * Constructor.
     * @param internals The file view
     */
    public CharacterLeftAction(Internals internals) {
        this.internals = internals;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        int cursorOffset = internals.getCursorOffset();
        int cursorLine = internals.getCursorLine();
        int cursorColumn = internals.getCursorColumn();

        if (cursorOffset == 0) {
            internals.setCursorMagic(0);
            internals.error("top");
            return;
        }

        internals.clearMessages();

        final int oldCursorLine = cursorLine;
        final int oldCursorCol = cursorColumn;

        final TokenizedFile tf = internals.getTokenizedFile();

        final boolean wasTab = tf.charAt(cursorOffset) == '\t';


        //
        // Move the cursorOffset left
        //
        cursorOffset--;
        if (tf.charAt(cursorOffset) == '\r') {
            cursorOffset--;
        }
        if ((tf.charAt(cursorOffset) == '\n')
                || wasTab) {
            if (tf.charAt(cursorOffset) == '\n') {
                cursorLine--;
                cursorOffset--;
                if (tf.charAt(cursorOffset) == '\n') {
                    cursorOffset++;
                }
            }
            final List tlv = tf.getLine(cursorLine);
            int so = ((LanguageToken) tlv.get(0)).getStartOffset();
            cursorColumn = 0;
            while (so < cursorOffset) {
                if (tf.charAt(so) == '\t') {
                    cursorColumn = ((cursorColumn / internals.getTabstop()) + 1)
                            * internals.getTabstop();
                } else {
                    cursorColumn++;
                }
                so++;
            }
        } else {
            cursorColumn--;
        }

        internals.setCursorOffset(cursorOffset);
        internals.setCursorLine(cursorLine);
        internals.setCursorColumn(cursorColumn);
        internals.setCursorMagic(cursorColumn);

        internals.adjustViewportAndDisplay(oldCursorLine, oldCursorCol);
    }
}
