/*
orion: Provides a set of source code browsing utilities.
Copyright (C) 2008  Steve Millington

$Id: CharacterRightAction.java 285 2011-07-19 12:37:42Z steve $

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.component.fileview;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

/**
 * File View Action - Move right 1 character.
 *
 * @author Steve Millington
 * @version $Id: CharacterRightAction.java 285 2011-07-19 12:37:42Z steve $
 */
public class CharacterRightAction extends AbstractAction {

    private final Internals internals;

    /**
     * Constructor.
     * @param internals The file view
     */
    public CharacterRightAction(Internals internals) {
        this.internals = internals;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        internals.clearMessages();
        final int oldCursorLine = internals.getCursorLine();
        final int oldCursorCol = internals.getCursorColumn();
        //
        // Move the cursorOffset right
        //
        try {
            final TokenizedFile tf = internals.getTokenizedFile();
            int nextCursorOffset = internals.getCursorOffset() + 1;
            int nextCursorLine = internals.getCursorLine();
            int nextCursorColumn = internals.getCursorColumn();

            if (tf.charAt(internals.getCursorOffset()) == '\n') {
                nextCursorOffset--;
            }

            if (tf.charAt(nextCursorOffset) == '\r') {
                nextCursorOffset++;
            }
            if (tf.charAt(nextCursorOffset) == '\n') {
                nextCursorLine++;
                nextCursorOffset++;
                nextCursorColumn = -1; // Adjusted by the next line!
            }
            //
            // Adjust the cursor column
            //
            if (tf.charAt(nextCursorOffset) == '\t') {
                nextCursorColumn++;
                nextCursorColumn =
                        (((nextCursorColumn / internals.getTabstop()) + 1)
                        * internals.getTabstop()) - 1;
            } else {
                nextCursorColumn++;
            }
            internals.setCursorOffset(nextCursorOffset);
            internals.setCursorLine(nextCursorLine);
            internals.setCursorColumn(nextCursorColumn);

            internals.setCursorMagic(nextCursorColumn);

            internals.adjustViewportAndDisplay(oldCursorLine, oldCursorCol);

        } catch (IndexOutOfBoundsException ex) {
            //
            // We went past the end of the buffer, so issue the message and
            // return.
            //
            internals.error("bottom");
        }
    }
}
