/*
orion: Provides a set of source code browsing utilities.
Copyright (C) 2008  Steve Millington

$Id: EndOfLineAction.java 285 2011-07-19 12:37:42Z steve $

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.component.fileview;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

/**
 * File View Action - Go to the end of the current line.
 *
 * @author Steve Millington
 * @version $Id: EndOfLineAction.java 285 2011-07-19 12:37:42Z steve $
 */
public class EndOfLineAction extends AbstractAction {

    private final Internals internals;

    /**
     * Constructor.
     * @param internals The file view
     */
    public EndOfLineAction(Internals internals) {
        this.internals = internals;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        internals.clearMessages();
        internals.goToEndOfLine();

        internals.setCursorMagic(internals.getCursorColumn());

        boolean displayAllNeeded = false;

        if (!internals.isCursorInViewportWidthwise()) {
            internals.adjustViewportHoriz();
            displayAllNeeded = true;
        }

        if (displayAllNeeded) {
            internals.displayAll();
        } else {
            internals.displayCursor();
        }

    }
}
