/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: LineDownAction.java 285 2011-07-19 12:37:42Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.component.fileview;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

/**
 * File View Action - Move down 1 line.
 *
 * @author Steve Millington
 * @version $Id: LineDownAction.java 285 2011-07-19 12:37:42Z steve $
 */
public class LineDownAction extends AbstractAction {
    private final Internals internals;
    /**
     * Constructor.
     * @param internals The file view
     */
    public LineDownAction(Internals internals) {
	this.internals = internals;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
	if (internals.getCursorLine() == internals.getLineCount() - 1) {
	    internals.error("bottom");
	    return;
	}
	internals.clearMessages();

	final int oldCursorLine = internals.getCursorLine();
	final int oldCursorCol = internals.getCursorColumn();

	internals.setCursorLine(internals.getCursorLine() + 1);

	//
	// move the cursorOffset and cursorColumn to the right place
	// Remember - cursorMagic is where we want to be!
	//
	internals.setCursorToMagic();

	internals.adjustViewportAndDisplay(oldCursorLine, oldCursorCol);
    }
}
