/*
orion: Provides a set of source code browsing utilities.
Copyright (C) 2008  Steve Millington

$Id: NextWordAction.java 285 2011-07-19 12:37:42Z steve $

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.component.fileview;

import java.awt.event.ActionEvent;

import java.util.List;

import javax.swing.AbstractAction;

import uk.co.ruelgnoj.orion.tokenizers.LanguageToken;
import uk.co.ruelgnoj.orion.tokenizers.TokenTypes;

/**
 * File View Action - Go to the start of the next (or maybe previous!) word.
 *
 * @author Steve Millington
 * @version $Id: NextWordAction.java 285 2011-07-19 12:37:42Z steve $
 */
public class NextWordAction extends AbstractAction {

    private final Internals internals;
    private final boolean back;

    /**
     * Constructor.
     * @param internals The file view
     * @param back Whether to search forwards or back
     */
    public NextWordAction(Internals internals, boolean back) {
        this.internals = internals;
        this.back = back;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        final TokenizedFile tf = internals.getTokenizedFile();

        internals.clearMessages();
        int startRow;
        int startIndex;
        List currentRowVec;
        int currentRow;
        int currentIndex;
        int endRow;
        int endIndex;

        final int oldCursorLine = internals.getCursorLine();
        final int oldCursorCol = internals.getCursorColumn();

        endRow = internals.getCursorLine();
        endIndex = internals.getCurrentTokenIndex();

        if (back) {
            //
            // Move to the prev token as the starting point
            //
            currentRowVec = tf.getLine(endRow);
            startIndex = endIndex - 1;
            startRow = endRow;
            if (startIndex < 0) {
                startRow--;
                if (startRow < 0) {
                    startRow = tf.getLineCount() - 1;
                }
                currentRowVec = tf.getLine(startRow);
                startIndex = currentRowVec.size() - 1;
            }
            if (startRow < 0) {
                startRow = tf.getLineCount() - 1;
            }
        } else {
            //
            // Move to the next token as the starting point
            //
            currentRowVec = tf.getLine(endRow);
            startIndex = endIndex + 1;
            startRow = endRow;
            if (startIndex >= currentRowVec.size()) {
                startIndex = 0;
                startRow++;
            }
            if (startRow >= tf.getLineCount()) {
                startRow = 0;
            }
        }

        currentRow = startRow;
        currentIndex = startIndex;
        currentRowVec = tf.getLine(startRow);

        //
        // loop until we reach the 'next' thingy
        //
        while ((endRow != currentRow) || (endIndex != currentIndex)) {
            //
            // Skip any blank lines!
            //
            while (currentRowVec.isEmpty()) {
                if (back) {
                    currentRow--;
                    if (currentRow < 0) {
                        currentRow = tf.getLineCount() - 1;
                        internals.warning("wrapped");
                    }
                } else {
                    currentRow++;
                    if (currentRow >= tf.getLineCount()) {
                        currentRow = 0;
                        internals.warning("wrapped");
                    }
                }
                currentRowVec = tf.getLine(currentRow);
                if (back) {
                    currentIndex = currentRowVec.size() - 1;
                } else {
                    currentIndex = 0;
                }
            }
            final LanguageToken tok = (LanguageToken) currentRowVec.get(
                    currentIndex);
            if ((tok.getType() != TokenTypes.TT_WHITESPACE)
                    && (tok.getType() != TokenTypes.TT_NEWLINE)) {
                break;
            }

            //
            // Go to the 'next' token
            //
            if (back) {
                currentIndex--;
                if (currentIndex < 0) {
                    currentRow--;
                    if (currentRow < 0) {
                        currentRow = tf.getLineCount() - 1;
                        internals.warning("wrapped");
                    }
                    currentRowVec = tf.getLine(currentRow);
                    currentIndex = currentRowVec.size() - 1;
                }
            } else {
                currentIndex++;
                if (currentIndex >= currentRowVec.size()) {
                    currentIndex = 0;
                    currentRow++;
                    if (currentRow >= tf.getLineCount()) {
                        currentRow = 0;
                        internals.warning("wrapped");
                    }
                }
                currentRowVec = tf.getLine(currentRow);
            }
        }


        //
        // Find out where the X psition of the start of the word is!
        //
        internals.setCursorOffset(
                ((LanguageToken) currentRowVec.get(currentIndex)).getStartOffset());
        internals.setCursorLine(currentRow);
        final int rowStartOffset =
                ((LanguageToken) currentRowVec.get(0)).getStartOffset();

        internals.setCursorColumn(0);
        for (int i = rowStartOffset; i < internals.getCursorOffset(); i++) {
            switch (tf.charAt(i)) {
                case '\t':
                    internals.setCursorColumn(
                            ((internals.getCursorColumn() / internals.getTabstop()) + 1)
                            * internals.getTabstop());
                    break;
                default:
                    internals.setCursorColumn(internals.getCursorColumn() + 1);
                    break;
            }
        }

        internals.setCursorMagic(internals.getCursorColumn());

        internals.adjustViewportAndDisplay(oldCursorLine, oldCursorCol);
    }
}
