/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: DefaultCursesModelTest.java 256 2008-07-15 15:54:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.cc;

import java.awt.Color;
import java.awt.Paint;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author steve
 */
public class DefaultCursesModelTest {
    
    private DefaultCursesModel _instance;
    public DefaultCursesModelTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() throws Exception {
        _instance = new DefaultCursesModel(20, 10);
    }

    @After
    public void tearDown() throws Exception {
        _instance = null;
    }

    @Test
    public void size() {
        System.out.println("size");
        int newWidth = 4;
        int newHeight = 5;
        _instance.setSize(newWidth, newHeight);
        assertEquals(5, _instance.getRows());
        assertEquals(4, _instance.getCols());
        
    } /* Test of setSize method, of class DefaultCursesModel. */

    @Test
    public void cursorVisible() {
        System.out.println("cursorVisible");
        _instance.setCursorVisible(false);
        assertEquals(false, _instance.isCursorVisible());
        _instance.setCursorVisible(true);
        assertEquals(true, _instance.isCursorVisible());
        _instance.setCursorVisible(false);
        assertEquals(false, _instance.isCursorVisible());
    } /* Test of setCursorVisible method, of class DefaultCursesModel. */

    @Test
    public void bg() {
        System.out.println("bg");
        final Color c = Color.RED;
        _instance.setBG(c);
        assertEquals(c, _instance.getBG());

    } /* Test of setBG method, of class DefaultCursesModel. */

    @Test
    public void fg() {
        System.out.println("fg");
        final Color c = Color.BLUE;
        _instance.setFG(c);
        assertEquals(c, _instance.getFG());
    } /* Test of setFG method, of class DefaultCursesModel. */

    @Test
    public void clear() {
        System.out.println("clear");
        _instance.print("Hello");
        _instance.clear();
        assertEquals(' ', _instance.getCharAt(0, 0));
    } /* Test of clear method, of class DefaultCursesModel. */

    @Test
    public void moveTo() {
        System.out.println("moveTo");
        _instance.moveTo(3, 4);
        assertEquals(3, _instance.getCursorX());
        assertEquals(4, _instance.getCursorY());
    } /* Test of moveTo method, of class DefaultCursesModel. */

    @Test
    public void print() {
        System.out.println("print");
        _instance.print("Hello");
        assertEquals('H', _instance.getCharAt(0, 0));
        assertEquals('e', _instance.getCharAt(1, 0));
    } /* Test of print method, of class DefaultCursesModel. */

    private void dumpInstance() {
        for (int y = 0; y < _instance.getCols(); y++) {
            System.out.print("*");
            for (int x = 0; x < _instance.getRows(); x++) {
            System.out.print(_instance.getCharAt(x,y));
            }
            System.out.print("*");
            System.out.println();
        }
    }
    @Test
    public void println() {
        System.out.println("println");
        _instance.setSize(10, 3);
        _instance.println("A");
         assertEquals('A', _instance.getCharAt(0, 0));
         assertEquals(0, _instance.getCursorX());
         assertEquals(1, _instance.getCursorY());
         _instance.println("B");
         _instance.println("C");
         assertEquals('B', _instance.getCharAt(0, 0));
         assertEquals('C', _instance.getCharAt(0, 1));
         assertEquals(' ', _instance.getCharAt(0, 2));
    } /* Test of println method, of class DefaultCursesModel. */

    @Test
    public void getForegroundAt() {
        System.out.println("getForegroundAt");
        Paint origFg = _instance.getFG();
        Color c = Color.BLUE;
        _instance.setFG(c);
        _instance.print("A");
        assertEquals(c, _instance.getForegroundAt(0, 0));
        assertEquals(origFg, _instance.getForegroundAt(1, 0));

    } /* Test of getForegroundAt method, of class DefaultCursesModel. */

    @Test
    public void getBackgroundAt() {
        System.out.println("getBackgroundAt");
        Paint origBg = _instance.getBG();
        Color c = Color.BLUE;
        _instance.setBG(c);
        _instance.print("A");
        assertEquals(c, _instance.getBackgroundAt(0, 0));
        assertEquals(origBg, _instance.getBackgroundAt(1, 0));

    } /* Test of getBackgroundAt method, of class DefaultCursesModel. */

    @Test
    public void isUnderLine() {
        System.out.println("isUnderLine");
        assertFalse(_instance.isUnderLine(0, 0));
        _instance.setUnderLine(true);
        _instance.print("A");
        assertTrue(_instance.isUnderLine(0, 0));
        assertFalse(_instance.isUnderLine(1, 0));
    } /* Test of isUnderLine method, of class DefaultCursesModel. */

    @Test
    public void setUnderLine() {
        System.out.println("setUnderLine");
        _instance.setUnderLine(false);
        assertEquals(false, _instance.isUnderline());
        _instance.setUnderLine(true);
        assertEquals(true, _instance.isUnderline());
        _instance.setUnderLine(false);
        assertEquals(false, _instance.isUnderline());

    } /* Test of setUnderLine method, of class DefaultCursesModel. */

    @Test
    public void scrollUp() {
        System.out.println("scrollUp");
        _instance.println("A");
        _instance.println("B");
        _instance.println("C");
        _instance.scrollUp(2);
        assertEquals('C', _instance.getCharAt(0, 0));
        assertEquals(' ', _instance.getCharAt(0, 1));
    } /* Test of scrollUp method, of class DefaultCursesModel. */

    @Test
    public void scrollDown() {
        System.out.println("scrollDown");
        _instance.setSize(10,5);
        _instance.print("A"); _instance.moveTo(0, 1);
        _instance.print("B"); _instance.moveTo(0, 2);
        _instance.print("C"); _instance.moveTo(0, 3);
        _instance.print("D"); _instance.moveTo(0, 4);
        _instance.print("E");
        _instance.scrollDown(2);
        assertEquals('C', _instance.getCharAt(0, 4));
        assertEquals('B', _instance.getCharAt(0, 3));
        assertEquals('A', _instance.getCharAt(0, 2));
        assertEquals(' ', _instance.getCharAt(0, 1));
    } /* Test of scrollDown method, of class DefaultCursesModel. */
    
}
