package uk.co.ruelgnoj.orion.component;

import de.jdemo.annotation.Demo;
import de.jdemo.extensions.SwingDemoCase;
import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.lang.reflect.InvocationTargetException;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLayer;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.plaf.LayerUI;
import org.junit.Test;
import uk.co.ruelgnoj.orion.cc.Curses;
import uk.co.ruelgnoj.orion.cc.DefaultCursesModel;

public class OrionComponentDemo extends SwingDemoCase {

    private final Object lock = new Object();

    @Test
    public void main() throws InterruptedException, InvocationTargetException {
        de.jdemo.swingui.DemoRunner.run(OrionComponentDemo.class);
        synchronized (lock) {
            lock.wait();
        }
    }

    @Demo
    public void curses() throws InterruptedException, InvocationTargetException {
        show(createCurses());
    }

    @Demo(description = "<h1>Orion Component - with Slow failure Model</h1>", categories = {"example", "public"})
    public void demoSlowFail() throws InterruptedException, InvocationTargetException {
        show(createOrion());
    }

    private JComponent createOrion() {
        final OrionComponent oc = new OrionComponent();
        oc.setModel(new SlowFailOrionDataModel());
        return oc;
    }

    private JButton mOverlayButton;

    private JComponent createCurses() {

        final WaitLayerUI layerUI = new WaitLayerUI();
        JPanel panel = createPanel();
        JLayer<JPanel> jlayer = new JLayer<>(panel, layerUI);

        final Timer stopper = new Timer(4000, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                layerUI.stop();
            }
        });
        stopper.setRepeats(false);

        mOverlayButton.addActionListener(
                new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent ae) {
                        layerUI.start();
                        if (!stopper.isRunning()) {
                            stopper.start();
                        }
                    }
                }
        );
        return jlayer;
    }

    private JPanel createPanel() {
        JPanel p1 = new JPanel(new BorderLayout());
        JPanel p = new JPanel();

        mOverlayButton = new JButton("Overlay...");
        p.add(mOverlayButton);
        final DefaultCursesModel m = new DefaultCursesModel(80, 24);
        final Curses curses = new Curses(m);
        curses.setFont(new Font("Courier", Font.PLAIN, 12));
        m.setBG(Color.WHITE);
        m.clear();

        
        final Paint c1 = new Color(80, 80, 200);
        final Paint c2 = new Color(80, 200, 0);
        final Paint c3 = new Color(0x7f, 0x00, 0x00);
        final Paint c4 = new Color(0x7f, 0x7f, 0x00);
        final Paint c5 = new Color(0x00, 0x7f, 0x7f);
        
        m.moveTo(25, 10);
        m.setFG(c1);
        m.print("This is a");
        m.setFG(c2);
        m.print(" demo ");
        m.setFG(c1);
        m.print("of");
        m.setFG(c3);
        m.print(" Overlaying.");
        m.setFG(c4);
        for (int i = 0; i < 24; i++) {
            m.setFG(c4);
            m.moveTo(76, i);
            m.print(fmt(i));
            m.setFG(c5);
            m.moveTo(0, i);
            m.print(fmt(i));
        }
        p1.add(BorderLayout.NORTH, p);
        p1.add(BorderLayout.CENTER, curses);

        return p1;
    }

    private String fmt(int n) {
        final String s = Integer.toString(n);
        if (s.length() == 4) {
            return s;
        } else {
            final String s1 = "000" + s;
            return s1.substring(s1.length() - 4, s1.length());
        }
    }

    private static class WaitLayerUI extends LayerUI<JPanel> implements ActionListener {

        private boolean mIsRunning;
        private boolean mIsFadingOut;
        private Timer mTimer;

        private int mAngle;
        private int mFadeCount;
        private int mFadeLimit = 15;

        @Override
        public void paint(Graphics g, JComponent c) {
            int w = c.getWidth();
            int h = c.getHeight();

            // Paint the view.
            super.paint(g, c);

            if (!mIsRunning) {
                return;
            }

            Graphics2D g2 = (Graphics2D) g.create();

            float fade = (float) mFadeCount / (float) mFadeLimit;
            // Gray it out.
            Composite urComposite = g2.getComposite();
            g2.setComposite(AlphaComposite.getInstance(
                    AlphaComposite.SRC_OVER, .5f * fade));
            g2.fillRect(0, 0, w, h);
            g2.setComposite(urComposite);

            // Paint the wait indicator.
            int s = Math.min(w, h) / 5;
            int cx = w / 2;
            int cy = h / 2;
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g2.setStroke(
                    new BasicStroke(s / 4, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
            g2.setPaint(Color.white);
            g2.rotate(Math.PI * mAngle / 180, cx, cy);
            for (int i = 0; i < 12; i++) {
                float scale = (11.0f - (float) i) / 11.0f;
                g2.drawLine(cx + s, cy, cx + s * 2, cy);
                g2.rotate(-Math.PI / 6, cx, cy);
                g2.setComposite(AlphaComposite.getInstance(
                        AlphaComposite.SRC_OVER, scale * fade));
            }

            g2.dispose();
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if (mIsRunning) {
                firePropertyChange("tick", 0, 1);
                mAngle += 3;
                if (mAngle >= 360) {
                    mAngle = 0;
                }
                if (mIsFadingOut) {
                    if (--mFadeCount == 0) {
                        mIsRunning = false;
                        mTimer.stop();
                    }
                } else if (mFadeCount < mFadeLimit) {
                    mFadeCount++;
                }
            }
        }

        public void start() {
            if (mIsRunning) {
                return;
            }

            // Run a thread for animation.
            mIsRunning = true;
            mIsFadingOut = false;
            mFadeCount = 0;
            int fps = 24;
            int tick = 1000 / fps;
            mTimer = new Timer(tick, this);
            mTimer.start();
        }

        public void stop() {
            mIsFadingOut = true;
        }

        @Override
        public void applyPropertyChange(PropertyChangeEvent pce, JLayer l) {
            if ("tick".equals(pce.getPropertyName())) {
                l.repaint();
            }
        }
    }
}
