/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defdm;

import java.io.IOException;
import java.util.List;
import uk.co.ruelgnoj.orion.defdm.impl.OrionRequest;
import uk.co.ruelgnoj.orion.defdm.impl.OrionRequestHandler;
import uk.co.ruelgnoj.orion.defdm.impl.RemoteRequestHandlerFactory;
import uk.co.ruelgnoj.orion.dm.OrionDataModel;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;
import uk.co.ruelgnoj.orion.dm.OrionFileEntry;

/**
 * This class is the entry into dealing with the data required for all types
 * of Orion product.
 * It not only accesses the Tags File, but also allows access to the raw file
 * data.
 *
 * @author Steve Millington
 * @version $Id: RemoteOrionDataModel.java 201 2008-01-14 17:27:42Z steve $
 */
public class HttpOrionDataModel implements OrionDataModel {

    private OrionRequestHandler rh;
    private String projectName;

    /**
     * Construct a Remote data model.
     *
     * @param url
     * @exception OrionDataModelException If a comms error occurs
     */
    public HttpOrionDataModel(String url) throws OrionDataModelException {
        try {
            rh = RemoteRequestHandlerFactory.createRequestHandler(url);
            projectName = url.substring(1 + url.lastIndexOf("/"));
        } catch (IOException ex) {
            throw new OrionDataModelException(ex);
        }
    }

    /** {@inheritDoc} */
    @Override
    @SuppressWarnings("unchecked")
    public synchronized List<String> getTags(String pattern) throws OrionDataModelException {
        final OrionRequest req = new OrionRequest(OrionRequest.Type.GET_MATCHING_TAGS, projectName, pattern);
        return (List<String>) rh.handleRequest(req);
    }

    /** {@inheritDoc} */
    @Override
    public CharSequence getFile(OrionFileEntry fileEntry) throws OrionDataModelException {
        return getFile(fileEntry.getFileName());
    }

    /** {@inheritDoc} */
    @Override
    public CharSequence getFile(String fileName) throws OrionDataModelException {
        //
        // Now, open the file and try to read the contents!
        //
        final OrionRequest req = new OrionRequest(OrionRequest.Type.GET_FILE, projectName, fileName);
        final Object resp = rh.handleRequest(req);
        final CharSequence dataBuffer = (CharSequence) resp;
        return dataBuffer;
    }

    /** {@inheritDoc} */
    @Override
    @SuppressWarnings("unchecked")
    public synchronized List<OrionFileEntry> getFiles(String tagString) throws OrionDataModelException {
        final OrionRequest req = new OrionRequest(OrionRequest.Type.GET_MATCHING_FILES, projectName, tagString);
        return (List<OrionFileEntry>) rh.handleRequest(req);
    }

    /** {@inheritDoc} */
    @Override
    public void destroy() {
        //
        // There is nothing to clean up!
        //
        rh.close();
    }
}
