/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defdm;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;

/**
 *
 * @author Steve Millington
 * @version $Id: JDBCOrionDataModel.java 219 2008-01-19 09:13:32Z steve $
 */
public final class JDBCOrionDataModel extends AbstractJDBCOrionDataModel {
    private String url_;
    private String projectName_;
    
    public JDBCOrionDataModel(String url) throws OrionDataModelException {
        determineProjectAndURL(url);

        init();
    }

    @Override
    protected Connection borrowConnection() throws SQLException {
        Connection ret;
        ret = DriverManager.getConnection(url_);
        ret.setAutoCommit(false);
        return ret;
    }

    @Override
    protected void returnConnection(Connection conn) {
        try {
            if (conn != null) {
                conn.commit();
                conn.close();
            }
        } catch (SQLException ex) {
            Logger.getLogger(JDBCOrionDataModel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void determineProjectAndURL(String baseURL) throws OrionDataModelException {
        // The project name is the last part of the URL, so we need to split things
        // up
        final int slashIndex = baseURL.lastIndexOf('/');
        if (slashIndex < 0) {
            throw new OrionDataModelException("URL Format error");
        }
        final int semiIndex = baseURL.indexOf(';');
        if (semiIndex >= slashIndex) {
            projectName_ = baseURL.substring(slashIndex + 1, semiIndex);
            baseURL = baseURL.substring(0, slashIndex) + baseURL.substring(semiIndex);
        } else {
            projectName_ = baseURL.substring(slashIndex);
            baseURL = baseURL.substring(0, slashIndex);
        }
        url_ = baseURL;
    }

    @Override
    protected String getProjectName() {
        return projectName_;
    }
}
