/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defdm.impl;

import java.io.Serializable;
import java.util.Arrays;

/**
 * A request for information from the Orion database.
 * @author Steve Millington
 * @version $Id: OrionRequest.java 201 2008-01-14 17:27:42Z steve $
 */
public class OrionRequest implements Serializable {
    public static enum Type {
        GET_FILE, /** Get a named file. */
        GET_MATCHING_FILES, /** Get the list of files for a given tag. */
        GET_MATCHING_TAGS /** Get the list of tags for a given partial tag. */
    };
    
    private final Type type;
    private Object[] requestObjects;

    /**
     * Constructor.
     * @param type The request type
     * @param objs The request parameters
     */
    public OrionRequest(Type type, Object... objs) {
	this.type = type;
	requestObjects = objs;
    }

    /**
     * Constructor.
     * @param type The request type
     */
    public OrionRequest(Type type) {
	this.type = type;
	requestObjects = null;
    }

    /**
     * Get the request type.
     * @return the type
     */
    public Type getType() {
    	return type;
    }

    /**
     * Set the request parameters.
     * @param objs The parameters
     */
    public void setRequestObject(Object... objs) {
	requestObjects = objs;
    }

    /**
     * Get the 'index'th request parameter.
     * @param index the index.
     * @return The parameter
     */
    public Object getRequestObject(int index) {
    	return requestObjects[index];
    }

    /**
     * Get the 'primary' request parameter.
     * @return The parameter
     */
    public Object getRequestObject() {
    	return getRequestObject(0);
    }

    @Override
    public String toString() {
    	return "OrionRequest[type="
		+ type
		+ ", ro="
		+ Arrays.toString(requestObjects)
		+ "]";
    }
}
