/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defdm.impl;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;

/**
 * A request handler that uses the HTTP protocol.
 *
 * @author Steve Millington
 * @version $Id: OrionRequestHandlerHTTP.java 201 2008-01-14 17:27:42Z steve $
 */
public final class OrionRequestHandlerHTTP implements OrionRequestHandler {

    private final URL url;

    /**
     * Constructor.
     *
     * @param url the url
     */
    private OrionRequestHandlerHTTP(URL url) {
        this.url = url;
    }

    /**
     * Create a new instance of the handler for the supplied applet.
     *
     * @param applet the applet
     * @return The handler
     * @throws MalformedURLException if there is a problem determining the
     * source of the required objects.
     */
    public static OrionRequestHandler newInstance(String baseUrl) throws MalformedURLException {
        String objectsServlet = null;
        if (objectsServlet == null) {
            objectsServlet = baseUrl;
            if (objectsServlet.endsWith("/")) {
                objectsServlet = objectsServlet.substring(
                        0, objectsServlet.length() - 1);
            }
            int idx;
            idx = objectsServlet.lastIndexOf('/');
            objectsServlet = objectsServlet.substring(
                    0, idx);
            objectsServlet += "/objects";
        }

        final OrionRequestHandler rh = new OrionRequestHandlerHTTP(new URL(objectsServlet));

        return rh;
    }

    @Override
    public Object handleRequest(OrionRequest req) throws OrionDataModelException {
        try {
            final URLConnection con = url.openConnection();

            con.setDoInput(true);
            con.setDoOutput(true);
            con.setRequestProperty("Content-Type",
                    "java-internal/" + req.getClass().getName());

            con.setUseCaches(false);

            try (ObjectOutputStream oos = new ObjectOutputStream(
                    new BufferedOutputStream(con.getOutputStream()))) {
                oos.writeObject(req);
                oos.flush();
            }
            final ObjectInputStream ois
                    = new ObjectInputStream(
                            new BufferedInputStream(con.getInputStream()));
            try {
                return ois.readObject();
            } catch (ClassNotFoundException e) {
                // Never happens?
                return null;
            }
        } catch (IOException e) {
            throw new OrionDataModelException(e);
        }
    }

    @Override
    public void close() {
    }

    @Override
    public String toString() {
        return "OrionRequestHandlerHTTP url=" + url.toString();
    }
}
