////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defdm.impl;

import uk.co.ruelgnoj.orion.dm.OrionDataModel;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;

/**
 * A request handler that gets its data from an orion data model of some sort.
 * @author Steve Millington
 * @version $Id: OrionRequestHandlerLocal.java 201 2008-01-14 17:27:42Z steve $
 */
public class OrionRequestHandlerLocal implements OrionRequestHandler {
    private final OrionDataModel odm;

    /**
     * Constructor.
     * @param odm The data model
     */
    public OrionRequestHandlerLocal(OrionDataModel odm) {
	this.odm = odm;
    }

    @Override
    public Object handleRequest(OrionRequest req) throws OrionDataModelException {
	final Object ret;

	switch (req.getType()) {
	    /*
	    case GET_PROPERTIES:
		{
		File file = (File)req.getRequestObject();
		Properties props = new Properties();
		try {
		    FileInputStream fis = new FileInputStream(file);
		    BufferedInputStream bis = new BufferedInputStream(fis);
		    props.load(fis);
		    bis.close();
		    fis.close();
		} catch (IOException e) {
		    System.err.println(
			    "IOError reading properties file " + file);
		}
		setIfNotSet(props, "orion.rows", "40");
		setIfNotSet(props, "orion.columns", "80");
		setIfNotSet(props, "orion.font", "Monospaced-plain-12");
		setIfNotSet(props, "orion.font.charWidth", "7");
		setIfNotSet(props, "orion.font.charHeight", "15");
		return props;
		}
	    case SET_PROPERTIES:
	    	List v = (List)req.getRequestObject();
		Properties props = (Properties)v.get(0);
		try {
		    FileOutputStream fos = new FileOutputStream((File)v.get(1));
		    BufferedOutputStream bos = new BufferedOutputStream(fos);
		    props.store(fos, "Orion Properties");
		    bos.close();
		    fos.close();
		} catch (IOException e) {
		    System.err.println(
			    "IOError writing properties file" + v.get(1));
		}
		return "";
		*/
	    case GET_FILE:
		final String src = (String)req.getRequestObject(1);
                // This 'tostring' is a hack and I want to remove it
                // but the general CharSequence interface is not serializable
		ret = odm.getFile(src).toString();
		break;
	    case GET_MATCHING_FILES:
		final String pattern0 = (String)req.getRequestObject(1);
		ret = odm.getFiles(pattern0);
		break;
	    case GET_MATCHING_TAGS:
		final String pattern1 = (String)req.getRequestObject(1);
		ret = odm.getTags(pattern1);
		break;
	    default:
		ret = null;
		break;
	}
	return ret;
    }

    @Override
    public void close() {
	if (odm != null) {
	    odm.destroy();
	}
    }
}
