/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defdm.impl;

import java.applet.Applet;
import java.io.IOException;

/**
 * A creator of remote request handlers.
 *
 * @author Steve Millington
 * @version $Id: RemoteRequestHandlerFactory.java 219 2008-01-19 09:13:32Z steve
 * $
 */
public final class RemoteRequestHandlerFactory {

    /**
     * Ensure nobody creates an instance.
     */
    private RemoteRequestHandlerFactory() {
    }

    /**
     * Create a request handler according to the parameters in the applet and
     * the current java version.
     * <p>
     * Note there were bugs in previous Java versions that mean that HTTP
     * communications were not possible. However in Java5 and greater these bugs
     * no longer exists, and so code to handle that has been removed.</p>
     *
     * @param applet The applet we are using for initialization parameters.
     * @throws IOException if the handler creation caused problems.
     * @return A handler.
     */
    public static OrionRequestHandler createRequestHandler(String baseUrl) throws IOException {
        return OrionRequestHandlerHTTP.newInstance(baseUrl);
    }
}
