/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.defproj;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import uk.co.ruelgnoj.orion.defproj.dto.CategoryDTO;
import uk.co.ruelgnoj.orion.defproj.dto.ProjectDTO;
import uk.co.ruelgnoj.orion.defproj.dto.TransferDTO;
import uk.co.ruelgnoj.orion.project.Category;
import uk.co.ruelgnoj.orion.project.Project;
import uk.co.ruelgnoj.orion.project.spi.ProjectFactoryImpl;

/**
 *
 * @author Steve Millington
 * @version $Id: JDBCProjectFactory.java 219 2008-01-19 09:13:32Z steve $
 */
public class HttpProjectFactory extends ProjectFactoryImpl {

    private static final Set<String> PROTOCOLS = Collections.unmodifiableSet(new HashSet<>(Arrays.asList("http", "https")));

    public HttpProjectFactory() {

    }

    @Override
    protected final Set<String> getProtocols() {
        return PROTOCOLS;
    }

    @Override
    public List<Category> createProjectsImpl(String url) {
        final Client client = ClientBuilder.newClient();
//        final WebTarget target = client.target(url).path("NotOrionWeb").path("index.html");
        final WebTarget target = client.target(url).path("rest").path("projects").path("2");
//        final WebTarget helloworldWebTargetWithQueryParam = target.queryParam("greeting", "Hi World!");
        final Invocation.Builder ss = target.request(MediaType.TEXT_PLAIN_TYPE);
        final Response response = ss.get();
        final String responseText = response.readEntity(String.class);

        final XStream stream = new XStream(new DomDriver());
        stream.processAnnotations(TransferDTO.class);
        stream.processAnnotations(CategoryDTO.class);
        stream.processAnnotations(ProjectDTO.class);
        final TransferDTO transfer = (TransferDTO) stream.fromXML(responseText);
        final List<Category> ret = new ArrayList<>();
        for (CategoryDTO cdto : transfer.getCategories()) {
            final List<Project> projects = new ArrayList<>();
            for (ProjectDTO pdto : cdto.getProjects()) {
                final HttpProject q = new HttpProject(pdto.getDescription(), url + "/" + pdto.getName());
                projects.add(q);
            }
            ret.add(new CategoryImpl(cdto.getDescription(), projects));
        }
        return ret;
//        StringTokenizer st = new StringTokenizer(responseText, "\n");
////        final List<String> projectNames = new ArrayList<>();
//        final Map<String, String> map = new HashMap<>();
//        while (st.hasMoreTokens()) {
//            final String line = st.nextToken();
//            final int equals = line.indexOf('=');
//            final String key = line.substring(0, equals);
//            final String value = line.substring(equals+1);
//            System.err.println("key=" + key);
//            System.err.println("value=" + value);
//            map.put(key, value);
////            final String projectName = st.nextToken();
////            projectNames.add(projectName);
//        }
//        return Collections.emptyList();
////        return projectNames.stream().map((a)->new HttpProject(url + "/" + a)).sorted((a,b)->a.getName().compareTo(b.getName())).collect(Collectors.toList());

    }
}
