/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: ProjectInfoTest.java 265 2008-07-17 09:42:46Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.defproj.dto;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import java.util.Collections;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import org.junit.Test;

/**
 *
 * @author Steve Millington
 * @version $Id: ProjectInfoTest.java 265 2008-07-17 09:42:46Z steve $
 */
public class TransferDTOTest {

    @Test
    public void saving() {
        final CategoryDTO cat = new CategoryDTO();
        cat.setDescription("Steve's ca\"t <b>a</b>");
        final ProjectDTO pi = new ProjectDTO();
        pi.setName("TheName");
        pi.setDescription("The description \"Hello\".");
        cat.setProjects(Collections.singletonList(pi));
        final TransferDTO od1 = new TransferDTO();
        od1.setCategories(Collections.singletonList(cat));
        final XStream stream = new XStream(new DomDriver());
        stream.processAnnotations(TransferDTO.class);
        final String s = stream.toXML(od1);
        System.out.println(s);
    }
    
    @Test
    public void noCategories() throws Exception {
        final XStream stream = new XStream(new DomDriver());
        stream.processAnnotations(TransferDTO.class);
        final StringBuilder sb = new StringBuilder();
        sb.append("<orion-transfer>");
        sb.append("</orion-transfer>");
        final TransferDTO tran = (TransferDTO) stream.fromXML(sb.toString());
        assertTrue(tran.getCategories().isEmpty());
    }
    
    @Test
    public void noProjects() throws Exception {
        final XStream stream = new XStream(new DomDriver());
        stream.processAnnotations(TransferDTO.class);
        stream.processAnnotations(CategoryDTO.class);
        stream.processAnnotations(ProjectDTO.class);
        final StringBuilder sb = new StringBuilder();
        sb.append("<orion-transfer>");
        sb.append("<category>");
        sb.append("<description>Steve</description>");
        sb.append("</category>");
        sb.append("</orion-transfer>");
        final TransferDTO tran = (TransferDTO) stream.fromXML(sb.toString());
        assertEquals(1, tran.getCategories().size());
        assertEquals("Steve", tran.getCategories().get(0).getDescription());
        assertTrue(tran.getCategories().get(0).getProjects().isEmpty());
    }
    
        
    @Test
    public void aProject() throws Exception {
        final XStream stream = new XStream(new DomDriver());
        stream.processAnnotations(TransferDTO.class);
        stream.processAnnotations(CategoryDTO.class);
        stream.processAnnotations(ProjectDTO.class);
        final StringBuilder sb = new StringBuilder();
        sb.append("<orion-transfer>");
        sb.append("<category>");
        sb.append("<description>Steve</description>");
        sb.append("<project name=\"p1\">");
        sb.append("<description>Something</description>");
        sb.append("</project>");
        sb.append("</category>");
        sb.append("</orion-transfer>");
        final TransferDTO tran = (TransferDTO) stream.fromXML(sb.toString());
        assertEquals(1, tran.getCategories().size());
        assertEquals("Steve", tran.getCategories().get(0).getDescription());
        assertEquals(1, tran.getCategories().get(0).getProjects().size());
        final ProjectDTO p = tran.getCategories().get(0).getProjects().get(0);
        assertEquals("p1", p.getName());
        assertEquals("Something", p.getDescription());
    }
}
