/*
 * -----------------------------------------------------------------------
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU Library General Public License as published
 *    by the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 * 
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------
 */
package uk.co.ruelgnoj.orion.dm;

import java.io.Serializable;

/**
 * An object representing a file entry in a orion database.
 */
public class OrionFileEntry implements Serializable {
    private static final long serialVersionUID = -7038869363171902404L;

    private final String fileName;
    private final int hitCount;

    private transient String path;
    private transient String name;

    /**
     * Constructor.
     * @param fileName
     * @param hitCount
     */
    public OrionFileEntry(String fileName, int hitCount) {
	this.fileName = fileName;
	this.hitCount = hitCount;
    }

    /**
     * Split out the various file parts.
     */
    private void factorize() {
	int slash = fileName.lastIndexOf('/');
	if (slash == -1) {
	    slash = fileName.lastIndexOf('\\');
	}
	this.path = fileName.substring(0, slash + 1);
	this.name = fileName.substring(slash + 1);
    }

    public String getName() {
	if (name == null) {
	    factorize();
	}
	return name;
    }

    public String getPath() {
	if (path == null) {
	    factorize();
	}
	return path;
    }

    /**
     * Get the name of the file.
     * @return the file name.
     */
    public String getFileName() {
	return fileName;
    }

    /**
     * Get the number of hits in the file.
     * @return The hit count.
     */
    public int getHitCount() {
	return hitCount;
    }
}
