/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: AbstractProjectServlet.java 248 2008-07-07 09:05:06Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.servlet;

import java.io.IOException;
import javax.servlet.ServletException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A base class for servlets dealing with projects.
 * @author Steve Millington
 * @version $Id: AbstractProjectServlet.java 248 2008-07-07 09:05:06Z steve $
 */
public abstract class AbstractProjectServlet extends HttpServlet {
    /**
     * Constructor.
     */
    public AbstractProjectServlet() {
    }
    
    @Override public void destroy() {
    }
    
    private ProjectManager getProjectManager() {
        return (ProjectManager)getServletContext().getAttribute("pm");
    }
    
    /**
     * Determine the project from the request.
     * This is determined from an early part of the path info.
     * Note that this method works in conjunction with getPathInProject().
     *
     * @return The project
     */
    private ProjectInfo getProjectInfo(HttpServletRequest req) {
        String projectName = null;
        String pathInfo = req.getPathInfo();
        if (pathInfo != null) {
            if (pathInfo.startsWith("/")) {
                pathInfo = pathInfo.substring(1);
            }
            final int slash = pathInfo.indexOf('/');
            if (slash > 0) {
                projectName = pathInfo.substring(0, slash);
            } else {
                projectName = pathInfo;
            }
        }
        
        final ProjectInfo pi;
        if (projectName != null) {
            pi = getProjectManager().getProject(projectName);
        } else {
            pi = null;
        }
        return pi;
    }
    
    private String getPathInProject(HttpServletRequest req) {
        String pathInfo = req.getPathInfo();
        if (pathInfo != null) {
            if (pathInfo.startsWith("/")) {
                pathInfo = pathInfo.substring(1);
            }
            int slash = pathInfo.indexOf('/');
            if (slash > 0) {
                String projectName = pathInfo.substring(0, slash);
                pathInfo = pathInfo.substring(projectName.length() + 1);
            } else {
                pathInfo = "";
            }
        }
        return pathInfo;
    }
    
    public abstract void doGet(HttpServletRequest req,
            HttpServletResponse res, ProjectInfo project, String pathInProject)
            throws ServletException, IOException;
    
    @Override
    public final void doGet(HttpServletRequest req,
            HttpServletResponse res)
            throws ServletException, IOException {
        final ProjectInfo pi = getProjectInfo(req);
        
        if ((pi == null) || !pi.isWellConfigured()) {
            res.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }
        
        final String pathInfo = getPathInProject(req);
        
        doGet(req, res, pi, pathInfo);
    }
    
    
}
