/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: OrionObjectsServlet.java 225 2008-03-24 18:39:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.servlet;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import uk.co.ruelgnoj.orion.defdm.impl.OrionRequest;

/**
 * The servlet servicing all request for serialized objects.
 *
 * @author Steve Millington
 * @version $Id: OrionObjectsServlet.java 225 2008-03-24 18:39:02Z steve $
 */
public class OrionObjectsServlet extends HttpServlet {

    private MasterOrionRequestHandler rh;

    /**
     * Constructor.
     */
    public OrionObjectsServlet() {
    }

    @Override
    public void init(ServletConfig cfg) throws ServletException {
        super.init(cfg);

        rh = new MasterOrionRequestHandler(getProjectManager());
    }

    @Override
    public void destroy() {
    }

    private ProjectManager getProjectManager() {
        return (ProjectManager) getServletContext().getAttribute("pm");
    }

    @Override
    public void doPost(HttpServletRequest req, HttpServletResponse res)            throws ServletException, IOException {
	//
        // Get the java objects for a request
        //
        final String contentType = req.getHeader("CONTENT-TYPE");
        if ((contentType == null)
                || (!contentType.startsWith("java-internal"))) {
            res.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

        final String ourCookieVal = ServletUtils.getCookieValue(req, res);

        ObjectOutputStream oos = null;
        try (ObjectInputStream ois = new ObjectInputStream(new BufferedInputStream(req.getInputStream()))) {
            final Object o;
            try {
                o = ois.readObject();
            } catch (ClassNotFoundException e) {
                log("", e);
                res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                return;
            }
            if (!(o instanceof OrionRequest)) {
                log("Received an object that was not an orion request");
                log(o.toString());
                res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                return;
            }

            final OrionRequest oreq = (OrionRequest) o;

            res.setHeader("Content-Type",
                    "java-internal/" + o.getClass().getName());

            oos = new ObjectOutputStream(
                    new BufferedOutputStream(res.getOutputStream()));

            final Object resp = rh.handleRequest(oreq);

            oos.writeObject(resp);
            oos.flush();
        } catch (IOException e) {
            log("", e);
            throw e;
        } catch (Throwable t) {
            log("", t);
            res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        } finally {
            if (oos != null) {
                oos.close();
            }
        }
    }
}
