/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: OrionServlet.java 225 2008-03-24 18:39:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;


import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;

/**
 * The servlet servicing orion page requests. These are of two types:
 * <ul>
 * <li>Requests for a base orion page. This gets forwarded to one of 2 jsps,
 * depending on whether the client is already configured or not.</li>
 * <li>Requests for source code in HTML format.</li>
 * </ul>
 *
 * @author Steve Millington
 * @version $Id: OrionServlet.java 225 2008-03-24 18:39:02Z steve $
 */
public class OrionServlet extends AbstractProjectServlet {

    private static final String DETERMINE_BROWSER_PROPS = "/internal/browser.jsp";
    private static final String ORION_PAGE = "/internal/orion.jsp";
    private static final String VIEW_CODE_PAGE = "/internal/view-code.jsp";
    private static final String HANDLE_THROWABLE = "/internal/handle-throwable.jsp";

    /**
     * Constructor.
     */
    public OrionServlet() {
    }

    @Override
    public void doGet(HttpServletRequest req, HttpServletResponse res, ProjectInfo project, String pathInProject) throws ServletException, IOException {

        //
        // If there is no 'extra' path info, then return the base HTML
        // page for the project.
        //
        if (pathInProject.length() == 0) {
            doGetBasePage(req, res, project);
        } else if (pathInProject.startsWith("file/")) {
            //
            // Get the (formatted) source content of a file.
            //
            try {
                final String name = pathInProject.substring(5, pathInProject.length());
                getSourceFile(req, res, project, name);
            } catch (OrionDataModelException e) {
                res.sendError(HttpServletResponse.SC_NOT_FOUND);
            }
        } else {
            res.sendError(HttpServletResponse.SC_NOT_FOUND);
        }
    }

    private void doGetBasePage(HttpServletRequest req,
            HttpServletResponse res,
            ProjectInfo pi) throws ServletException, IOException {

        try {
            final String ourCookieVal = ServletUtils.getCookieValue(req, res);
            final RequestDispatcher rd;
            rd = getServletContext().getRequestDispatcher(ORION_PAGE);
            req.setAttribute("cookie", ourCookieVal);
            req.setAttribute("title", pi.getDescription());
            req.setAttribute("project", pi);
            req.setAttribute("locale", ServletUtils.getPrimaryLocale(req).toString());
            rd.forward(req, res);
        } catch (Throwable t) {
            req.setAttribute("throwable", t);
            getServletContext().getRequestDispatcher(HANDLE_THROWABLE).forward(req, res);
        }
    }

    private void getSourceFile(HttpServletRequest req,
            HttpServletResponse res,
            ProjectInfo pi,
            String fileName) throws OrionDataModelException, IOException, ServletException {
        final CharSequence chars = pi.getDataModel().getFile(fileName);

        final StringWriter sw = new StringWriter();
        try (PrintWriter pw = new PrintWriter(sw)) {
            pi.getSourceFormatter().format(fileName, chars, pw);
        }

        req.setAttribute("title", fileName);
        req.setAttribute("content", sw.toString());
        getServletContext().getRequestDispatcher(VIEW_CODE_PAGE).forward(req, res);
    }
}
