/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: ProjectSetupContextListener.java 271 2009-04-15 21:53:25Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uk.co.ruelgnoj.orion.servlet;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.EnumSet;
import java.util.Set;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.servlet.DispatcherType;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.FilterRegistration;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebListener;
import uk.co.ruelgnoj.orion.product.Product;

/**
 * The servlet initializing all of the environment for the other servlets and
 * JSPs used by Orion.
 *
 * @author Steve Millington
 * @version $Id: ProjectSetupContextListener.java 271 2009-04-15 21:53:25Z steve
 * $
 */
@WebListener
public class ProjectSetupContextListener implements ServletContextListener {

    /**
     * Constructor.
     */
    public ProjectSetupContextListener() {
    }

    @Override
    public void contextInitialized(ServletContextEvent evt) {
        final ServletContext ctx = evt.getServletContext();
        try {
            final Context initCtx = new InitialContext();
            final String orionXML = ServletUtils.expand((String) initCtx.lookup("java:comp/env/orionxml"));

            // Set 'orion.version'
            ctx.setAttribute("orion.version",
                    Product.getBundle().getString("orion.version"));
            ctx.setAttribute("orion.productname",
                    Product.getBundle().getString("orion.productname"));

            // Set 'pm' - the ProjectManager
            ctx.log("Configuration XML is " + orionXML);
            final ProjectManager projectManager = new ProjectManager(new File(orionXML));
            ctx.setAttribute("pm", projectManager);

            debugProjectManager(ctx, projectManager);

            ctx.log("Project Setup Complete");
        } catch (NameNotFoundException e) {
            ctx.log("Required env entry " + e.getMessage());
            setupFailed(ctx);
        } catch (NamingException e) {
            ctx.log("ProjectA Initialisation Failed", e);
            setupFailed(ctx);
        }
    }

    private void setupFailed(ServletContext ctx) {
        final FilterRegistration.Dynamic i = ctx.addFilter("SetupFailed", SetupFailedFilter.class);
        i.addMappingForUrlPatterns(EnumSet.allOf(DispatcherType.class), true, "*");
    }

    public static class SetupFailedFilter implements Filter {

        @Override
        public void init(FilterConfig filterConfig) throws ServletException {
        }

        @Override
        public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
            response.setContentType("text/html");
            final PrintWriter pw = response.getWriter();
            pw.println("<html>");
            pw.println("<body>");
            pw.println("<p>Context setup failed</p>");
            pw.println("</body>");
            pw.println("</html>");
        }

        @Override
        public void destroy() {
        }

    }

    private void debugProjectManager(ServletContext ctx, ProjectManager pm) {
        final Set<ProjectInfo> projects = pm.getProjects();
        if (projects.isEmpty()) {
            ctx.log("No projects available");
        } else {
            // Dump out the projects for debug purposes
            pm.getProjects().forEach((pi) -> ctx.log("Project: " + pi.getName()));
        }
    }

    @Override
    public void contextDestroyed(ServletContextEvent evt) {
        final ServletContext ctx = evt.getServletContext();
        ctx.setAttribute("pm", null);
        ctx.setAttribute("cch", null);
    }
}
