/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: ManagedJDBCOrionDataModel.java 225 2008-03-24 18:39:02Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.servlet.dm;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import uk.co.ruelgnoj.orion.defdm.AbstractJDBCOrionDataModel;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;

/**
 *
 * @author Steve Millington
 * @version $Id: ManagedJDBCOrionDataModel.java 225 2008-03-24 18:39:02Z steve $
 */
public class ManagedJDBCOrionDataModel extends AbstractJDBCOrionDataModel {

    private String url_;
    private String projectName_;

    public ManagedJDBCOrionDataModel(String url) throws OrionDataModelException {
        determineProjectAndURL(url);

        init();
    }

    @Override
    protected Connection borrowConnection() throws SQLException {
        try {
            final InitialContext ic = new InitialContext();
            final DataSource ds = (DataSource)ic.lookup(url_);
            final Connection ret = ds.getConnection();
            return ret;
        } catch (NamingException e) {
            throw new SQLException(e);
        }
    }

    @Override
    protected void returnConnection(Connection conn) {
        try {
            if (conn != null) {
                conn.close();
            }
        } catch (SQLException ex) {
            Logger.getLogger(ManagedJDBCOrionDataModel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void determineProjectAndURL(String baseURL) throws OrionDataModelException {
        // The project name is the last part of the URL, so we need to split things
        // up
        final int slashIndex = baseURL.lastIndexOf('/');
        if (slashIndex < 0) {
            throw new OrionDataModelException("URL Format error");
        }
        final int semiIndex = baseURL.indexOf(';');
        if (semiIndex >= slashIndex) {
            projectName_ = baseURL.substring(slashIndex + 1, semiIndex);
            baseURL = baseURL.substring(0, slashIndex) + baseURL.substring(semiIndex);
        } else {
            projectName_ = baseURL.substring(slashIndex);
            baseURL = baseURL.substring(0, slashIndex);
        }
        url_ = baseURL;
    }

    @Override
    protected String getProjectName() {
        return projectName_;
    }
}
