package uk.co.ruelgnoj.orion.servlet.rest;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import java.util.StringJoiner;
import java.util.stream.Collectors;
import javax.servlet.ServletContext;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import uk.co.ruelgnoj.orion.defproj.dto.CategoryDTO;
import uk.co.ruelgnoj.orion.defproj.dto.ProjectDTO;
import uk.co.ruelgnoj.orion.defproj.dto.TransferDTO;
import uk.co.ruelgnoj.orion.servlet.ProjectManager;

/**
 *
 * @author steve
 */
@Path("/projects")
public class FirstService {

    @Context
    private ServletContext context;

    @GET
    // The Java method will produce content identified by the MIME Media
    // type "text/plain"
    @Produces("text/plain")
    public String archaic() {
        final ProjectManager pm = getProjectManager();
        final StringJoiner sj = new StringJoiner("\n");
        if (pm != null) {
            pm.getWellConfiguredProjects().forEach((a) -> sj.add(a.getName()));
        }
        return sj.toString();
    }

    @Path("/2")
    @GET
    // The Java method will produce content identified by the MIME Media
    // type "text/plain"
    @Produces("text/plain")
    public String nextTry() {
        final ProjectManager pm = getProjectManager();
        final TransferDTO t = new TransferDTO();
        if (pm != null) {
            t.setCategories(pm.getCategories().stream().map((cat) -> {
                final CategoryDTO cdto = new CategoryDTO();
                cdto.setDescription(cat.getDescription());
                cdto.setProjects(cat.getProjects().stream().map((p) -> {
                    final ProjectDTO pdto = new ProjectDTO();
                    pdto.setName(p.getName());
                    pdto.setDescription(p.getDescription());
                    return pdto;
                }).collect(Collectors.toList()));
                return cdto;
            }).collect(Collectors.toList()));
        }
        final XStream stream = new XStream(new DomDriver());
        stream.processAnnotations(TransferDTO.class);
        stream.processAnnotations(CategoryDTO.class);
        stream.processAnnotations(ProjectDTO.class);
        return stream.toXML(t);
    }

    private ProjectManager getProjectManager() {
        return (ProjectManager) context.getAttribute("pm");
    }
}
