/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: ProjectManagerTest.java 265 2008-07-17 09:42:46Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.servlet;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import org.junit.Test;
import static org.junit.Assert.*;


public class ProjectManagerTest {

    @Test(expected=NullPointerException.class) 
    public void doNull() {
            new ProjectManager((File) null);
    }

    @Test
    public void empty() throws IOException {
        final ProjectManager pm = new ProjectManager(getResourceInputStream(
                "TestProjectManager-testEmpty.xml"));
        assertTrue(pm.getProjects().isEmpty());
        assertNull(pm.getProject("does-not-exist"));
    }

    @Test
    public void single() throws IOException {
        final ProjectManager pm = new ProjectManager(getResourceInputStream(
                "TestProjectManager-testSingle.xml"));
        assertEquals(1, pm.getProjects().size());        
        ProjectInfo pi = pm.getProject("name-TS1");
        assertEquals("name-TS1", pi.getName());
        assertEquals("description-TS1", pi.getDescription());
        assertEquals("foo:database-TS1", pi.getDatabase());
        assertFalse(pi.isWellConfigured());
    }

    private InputStream getResourceInputStream(String name) {
        String res = "/uk/co/ruelgnoj/orion/servlet/" + name;
        return getClass().getResourceAsStream(res);
    }
}

