/*
 orion: Provides a set of source code browsing utilities.
 Copyright (C) 2008  Steve Millington

 $Id: ServletUtilsTest.java 271 2009-04-15 21:53:25Z steve $

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package uk.co.ruelgnoj.orion.servlet;

import com.mockrunner.mock.web.MockHttpServletRequest;
import com.mockrunner.mock.web.MockHttpServletResponse;
import java.util.HashMap;
import java.util.Locale;
import javax.servlet.http.Cookie;
import java.util.Map;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author steve
 */
public class ServletUtilsTest {
    
    /**
     * Test of getPrimaryLocale method, of class uk.co.ruelgnoj.orion.servlet.ServletUtils.
     */
    @Test
    public void getPrimaryLocale() {
        MockHttpServletRequest req = new MockHttpServletRequest();
        
        Locale result = ServletUtils.getPrimaryLocale(req);
        assertEquals(Locale.getDefault(), result);
        
        req = new MockHttpServletRequest();
        req.addHeader("Accept-Language", "FOO");
        result = ServletUtils.getPrimaryLocale(req);
        assertEquals(new Locale("foo"), result);
        
        req = new MockHttpServletRequest();
        req.addHeader("Accept-Language", "fr");
        result = ServletUtils.getPrimaryLocale(req);
        assertEquals(Locale.FRENCH, result);


    }

    /**
     * Test of htmlSafe method, of class uk.co.ruelgnoj.orion.servlet.ServletUtils.
     */
    @Test
    public void htmlSafe() {
        final String buffer = "&<>\"\n\rfoo";
        
        final String expResult = "&amp;&lt;&gt;&quot;foo";
        final String result = ServletUtils.htmlSafe(buffer);
        assertEquals(expResult, result);
    }


    /**
     * Test of expand method, of class uk.co.ruelgnoj.orion.servlet.ServletUtils.
     */
    @Test
    public void expand() {
        final Map<Object, Object> values = new HashMap<>();
        values.put("key1", "value7");
        values.put("key4", "a${key1}b");
        values.put("badkey1", "a${badkey2}b");
        values.put("badkey2", "c${badkey1}d");

        assertEquals("", ServletUtils.expand(values, ""));
        assertEquals("value7", ServletUtils.expand(values, "${key1}"));
        assertEquals("${key2}", ServletUtils.expand(values, "${key2}"));
        assertEquals("cavalue7bd", ServletUtils.expand(values, "c${key4}d"));
        assertEquals("cavalue7bdvalue7", ServletUtils.expand(values, "c${key4}d${key1}"));
        assertEquals("$", ServletUtils.expand(values, "$"));
        assertEquals("${", ServletUtils.expand(values, "${"));
        assertEquals("${aa", ServletUtils.expand(values, "${aa"));
        assertEquals("ac${badkey1}db", ServletUtils.expand(values, "${badkey1}"));
        assertEquals("ac${badkey1}dbac${badkey1}db", ServletUtils.expand(values, "${badkey1}${badkey1}"));
    }
    
}
