////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: OrionTagsTask.java 285 2011-07-19 12:37:42Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tags;

import java.util.Arrays;
import java.util.Iterator;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.taskdefs.MatchingTask;

/**
 * An <a href="http://ant.apache.org">Ant</a> task for running the Orion
 * tagfile generation process.
 * @author Steve Millington
 * @version $Id: OrionTagsTask.java 285 2011-07-19 12:37:42Z steve $
 */
public class OrionTagsTask extends MatchingTask {
    private File dbFile;
    private File sourceDir;
    private boolean verbose;
    private boolean silent;

    public OrionTagsTask() {
    }

    @Override
    public void init() {
	verbose = false;
	silent = false;
	sourceDir = null;
	dbFile = null;
    }

    public void setFile(File f) {
	dbFile = f;
    }

    public void setVerbose(boolean b) {
	if (verbose != b) {
	    verbose = b;
	    if (verbose) {
		silent = false;
	    }
	}
    }

    public void setSilent(boolean b) {
	if (silent != b) {
	    silent = b;
	    if (silent) {
		verbose = false;
	    }
	}
    }

    public void setSource(File f) {
	sourceDir = f;
    }

    @Override
    public void execute() throws BuildException {
	if (sourceDir == null) {
	    final String s = getProject().getProperty("basedir");
	    sourceDir = new File(s);
	}
	log("Creating " + dbFile.getAbsolutePath());
	final OrionTags ot = new OrionTags();
	try {
	    ot.setSourceDirectory(sourceDir);
	    ot.setDBFile(dbFile);
	    ot.setVerbose(verbose);
	    ot.setSilent(silent);

	    DirectoryScanner ds = getDirectoryScanner(sourceDir);
	    ds.scan();
	    final String[] files = ds.getIncludedFiles();
	    Arrays.sort(files);

	    //
	    // Generate the DB based on an iterator of the files to use.
	    //
	    ot.generate(new Iterator<File>() {
		private int index = 0;
                @Override
		public boolean hasNext() {
		    return index != files.length;
		}
                @Override
		public File next() {
		    final File ret = new File(sourceDir, files[index]);
		    index++;
		    return ret;
		}
                @Override
		public void remove() {
		    throw new UnsupportedOperationException();
		}
	    });

	} catch (TagDatabaseException | IllegalStateException e) {
	    throw new BuildException(e);
	}
    }
}
