////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: Parser.java 182 2008-01-06 10:56:16Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tags;

import java.nio.CharBuffer;

import uk.co.ruelgnoj.orion.tokenizers.LanguageToken;
import uk.co.ruelgnoj.orion.tokenizers.LanguageTokenizer;

/**
 * The parser. It doesn't actually do much parsing, but refers nearly everything
 * to the tokenizer. The main role of this class is to put the tokens from
 * the tokenizer into the database.
 *
 * @author Steve Millington
 * @version $Id: Parser.java 182 2008-01-06 10:56:16Z steve $
 */
public class Parser {
    private final TagDatabase db;
    private LanguageTokenizer tokenizer;

    /**
     * Constructor.
     * @param db The target database.
     */
    public Parser(TagDatabase db) {
	this.db = db;
    }

    /**
     * Set the tokenizer to use. This gets set multiple times in the life of
     * the parser.
     * @param tokenizer the tokenizer
     */
    public void setTokenizer(LanguageTokenizer tokenizer) {
	this.tokenizer = tokenizer;
    }

    /**
     * Parse a block of data using the current parser.
     * @param data The data
     * @param len The amount of data
     * @throws TagDatabaseException if the DB cannot be written to
     */
    public void parse(char[] data, int len) throws TagDatabaseException {
	parse(CharBuffer.wrap(data, 0, len));
    }

    /**
     * Parse a block of data using the current parser.
     * @param data The data
     * @throws TagDatabaseException if the DB cannot be written to
     */
    public void parse(CharSequence data) throws TagDatabaseException {
	tokenizer.setBuffer(data);

	while (tokenizer.hasNext()) {
	    final LanguageToken tok = tokenizer.next();
	    if (tok.getType().isIndexable()) {
		db.add(tok.asString(data));
	    }
	}
    }
}
