////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: TagDatabase.java 182 2008-01-06 10:56:16Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tags;

import java.io.PrintWriter;

/**
 * A database for storing tag information.
 * @author Steve Millington
 * @version $Id: TagDatabase.java 182 2008-01-06 10:56:16Z steve $
 */
public interface TagDatabase {

    /**
     * Set the name of the file currently being processed.
     * @param fn The name of the file.
     * @throws TagDatabaseException if a problem occurs.
     */
    void setFile(String fn) throws TagDatabaseException;

    /**
     * Define a tag as being used by the current file.
     * @param newTag The name of the tag.
     * @throws TagDatabaseException if a problem occurs.
     */
    void add(String newTag) throws TagDatabaseException;

    /**
     * Commit the database.
     * @throws TagDatabaseException if a problem occurs.
     */
    void commit() throws TagDatabaseException;

    /**
     * Print some statistics.
     * @param pw Where to print to.
     */
    void printStats(PrintWriter pw);
}
