////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: CharUtils.java 88 2006-12-01 22:39:47Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers;

/**
 * A set of utility Character routines.
 * @author Steve Millington
 * @version $Id: CharUtils.java 88 2006-12-01 22:39:47Z steve $
 */
public final class CharUtils {

    /**
     * Private constructor so that nobody creates one.
     */
    private CharUtils() {
    }

    /**
     * Is the supplied character a digit 0-9.
     * @param c The character to test
     * @return true if the character is a digit
     */
    public static boolean isDigit(char c) {
	return (c >= '0') && (c <= '9');
    }

    /**
     * Is the supplied character a whitespace character (space or tab).
     * @param c The character to test
     * @return true if the character is a whitespace character
     */
    public static boolean isWhite(char c) {
	return (c == ' ') || (c == '\t');
    }

    /**
     * Is the supplied character a hash character.
     * @param c The character to test
     * @return true if the character is a hash character
     */
    public static boolean isHash(char c) {
	return c == '#';
    }

    /**
     * Is the supplied character an alpha character (A-Z,a-z).
     * @param c The character to test
     * @return true if the character is an alpha character
     */
    public static boolean isAlpha(char c) {
	return ((c >= 'A') && (c <= 'Z'))
		|| ((c >= 'a') && (c <= 'z'));
    }

    /**
     * Is the supplied character an alphanumeric character.
     * @param c The character to test
     * @return true if the character is an alphanumeric character
     */
    public static boolean isAlNum(char c) {
	return isAlpha(c) || isDigit(c);
    }

    /**
     * Is the supplied character an alphanumeric character or an underscore.
     * @param c The character to test
     * @return true if the character is an alphanumeric character or underscore
     */
    public static boolean isAlNumUnderscore(char c) {
	return isAlNum(c) || (c == '_');
    }

    /**
     * Is the supplied character an alpha character or an underscore.
     * @param c The character to test
     * @return true if the character is an alpha character or underscore
     */
    public static boolean isAlphaUnderscore(char c) {
	return isAlpha(c) || (c == '_');
    }
}
