////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: TokenTypeImpl.java 285 2011-07-19 12:37:42Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers;

import java.util.ResourceBundle;

/**
 * An implementation of a token type.
 * <p>This class is not designed for direct use outside of this package.
 *
 * @author Steve Millington
 * @version $Id: TokenTypeImpl.java 285 2011-07-19 12:37:42Z steve $
 */
public class TokenTypeImpl implements TokenType {

    private static final String RBNAME =
            "uk.co.ruelgnoj.orion.tokenizers.resources";
    private String name;
    private boolean indexable;

    /**
     * Constructor.
     * @param name The name of the token type
     */
    TokenTypeImpl(String name) {
        this(name, false);
    }

    /**
     * Constructor.
     * @param name The name of the token type
     * @param indexable If this type oftoken can be indexed.
     */
    TokenTypeImpl(String name, boolean indexable) {
        this.name = name;
        this.indexable = indexable;
    }

    @Override
    public boolean equals(Object o) {
        if (!(o instanceof TokenType)) {
            return false;
        }
        final TokenType that = (TokenType) o;
        return this.getName().equals(that.getName());
    }

    @Override
    public int hashCode() {
        return name.hashCode();
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getDisplayName() {
        return ResourceBundle.getBundle(RBNAME).getString(
                "tokentype." + name + ".desc");
    }

    @Override
    public boolean isIndexable() {
        return indexable;
    }

    @Override
    public String toString() {
        return "TokenTypeImpl[" + name + "]";
    }
}
