////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: TokenTypes.java 88 2006-12-01 22:39:47Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers;

/**
 * The constant set of all token types.
 *
 * @author Steve Millington
 * @version $Id: TokenTypes.java 88 2006-12-01 22:39:47Z steve $
 */
public interface TokenTypes {

    /** A language keyword. */
    TokenType TT_KEYWORD = new TokenTypeImpl("keyword");

    /** A language word (e.g variable name). */
    TokenType TT_WORD = new TokenTypeImpl("word", true);

    /** A CICS API command. */
    TokenType TT_CICS_API = new TokenTypeImpl("cicsapi");

    /** An SQL API command. */
    TokenType TT_SQL_API = new TokenTypeImpl("sqlapi");

    /** A numeric value. */
    TokenType TT_NUMBER = new TokenTypeImpl("number");

    /** Whitepace (tabs and spaces). */
    TokenType TT_WHITESPACE = new TokenTypeImpl("whitespace");

    /** Newlines. */
    TokenType TT_NEWLINE = new TokenTypeImpl("newline");

    /** Strings. */
    TokenType TT_STRING = new TokenTypeImpl("string");

    /** Individual language characters. */
    TokenType TT_CHAR = new TokenTypeImpl("character");

    /** Comments. */
    TokenType TT_COMMENT = new TokenTypeImpl("comment");

    /** Punctuation. */
    TokenType TT_PUNC = new TokenTypeImpl("punctuation");

    /** Line numbers. */
    TokenType TT_LINENUM = new TokenTypeImpl("linenumber");

    /** Plain text. */
    TokenType TT_PLAINTEXT = new TokenTypeImpl("plaintext");
}
