////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: CICSCobolLanguageTokenizer.java 232 2008-06-04 15:31:24Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.ResourceBundle;
import uk.co.ruelgnoj.orion.tokenizers.LanguageTokenizer;

/**
 * A tokenizer for CICS COBOL language files.
 * <p>This supports both <code>EXEC CICS</code> and <code>EXEC SQL</code>
 * code.
 */
public class CICSCobolLanguageTokenizer extends DelegatingLanguageTokenizer {

    /** @see uk.co.ruelgnoj.orion.tokenizers.impl.DelegatingLanguageTokenizer */
    public CICSCobolLanguageTokenizer() {
    }

    @Override
    public LanguageTokenizer createDelegate() {
	CobolLanguageTokenizer clt = new CobolLanguageTokenizer();
	CICSMixIn mixinCICS = new CICSMixIn(clt, CICSMixIn.Flavour.COBOL);
	SQLMixIn mixinSQL = new SQLMixIn(mixinCICS, SQLMixIn.Flavour.COBOL);
	return mixinSQL;
    }

    private static final String RBNAME =
		"uk.co.ruelgnoj.orion.tokenizers.resources";

    @Override
    public String getDescription() {
	return ResourceBundle.getBundle(RBNAME).getString(
		"lang.cicscobol.desc");
    }

    @Override
    public String getLanguage() {
    	return "cicscobol";
    }
}
