////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: CSharpLanguageTokenizer.java 226 2008-05-11 15:10:59Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * A tokenizer for C-Sharp source files.
 *
 * @author Steve Millington
 * @version $Id: CSharpLanguageTokenizer.java 226 2008-05-11 15:10:59Z steve $
 */
public class CSharpLanguageTokenizer extends CLanguageTokenizer {
    //
    // The reserved words is a static table, that is initialized by the static
    // initializer at the bottom of this file. It is returned to the superclass
    // when it asks for them.
    //
    private static final Collection<String> RESERVED_WORDS;
    private static final Collection<String> PP_RESERVED_WORDS;

    private static final String[] WORDS = {
	"abstract",
	"as",
	"base",
	"bool",
	"break",
	"byte",
	"case",
	"catch",
	"char",
	"checked",
	"class",
	"const",
	"continue",
	"decimal",
	"default",
	"delegate",
	"do",
	"double",
	"else",
	"enum",
	"event",
	"explicit",
	"extern",
	"false",
	"finally",
	"fixed",
	"float",
	"for",
	"foreach",
	"goto",
	"if",
	"implicit",
	"in",
	"int",
	"interface",
	"internal",
	"is",
	"lock",
	"long",
	"namespace",
	"new",
	"null",
	"object",
	"operator",
	"out",
	"override",
	"params",
	"private",
	"protected",
	"public",
	"readonly",
	"ref",
	"return",
	"sbyte",
	"sealed",
	"short",
	"sizeof",
	"stackalloc",
	"static",
	"string",
	"struct",
	"switch",
	"this",
	"throw",
	"true",
	"try",
	"typeof",
	"uint",
	"ulong",
	"unchecked",
	"unsafe",
	"ushort",
	"using",
	"virtual",
	"void",
	"volatile",
	"while"
    };

    private static final String[] PPWORDS = {
	"if",
	"else",
	"elif",
	"endif",
	"define",
	"undef",
	"warning",
	"error",
	"line",
	"region",
	"endregion"
    };

    static {
	Set<String> set = new HashSet<>(Arrays.asList(WORDS));
	RESERVED_WORDS = Collections.unmodifiableSet(set);
	set = new HashSet<>(Arrays.asList(PPWORDS));
	PP_RESERVED_WORDS = Collections.unmodifiableSet(set);
    }

    /**
     * Constructor.
     * @see uk.co.ruelgnoj.orion.tokenizers.impl.CLanguageTokenizer
     */
    public CSharpLanguageTokenizer() {
    }

    @Override
    protected Collection<String> getReservedWords() {
	return RESERVED_WORDS;
    }

    @Override
    protected Collection<String> getPreprocessorReservedWords() {
	return PP_RESERVED_WORDS;
    }

    @Override
    public String getDescription() {
	return "C#";
    }

    @Override
    public String getLanguage() {
	return "cs";
    }
}
