////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: CppLanguageTokenizer.java 285 2011-07-19 12:37:42Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * A tokenizer for C++ source files.
 *
 * @author Steve Millington
 * @version $Id: CppLanguageTokenizer.java 285 2011-07-19 12:37:42Z steve $
 */
public class CppLanguageTokenizer extends CLanguageTokenizer {
    //
    // The reserved words is a static table, that is initialized by the static
    // initializer. It is returned to the superclass when it asks for them.
    //
    private static final Collection<String> reservedWords;
    private static final String[] WORDS = {
        "catch",
	"class",
        "delete",
        "except",
        "finally",
        "friend",
        "inline",
        "new",
        "operator",
        "private",
        "protected",
        "public",
        "template",
        "this",
        "throw",
        "try",
        "virtual"
    };
    static {
	reservedWords = new HashSet<>(Arrays.asList(WORDS));
    }


    /**
     * Constructor.
     * @see uk.co.ruelgnoj.orion.tokenizers.impl.CLanguageTokenizer
     */
    public CppLanguageTokenizer() {
    }

    @Override
    protected Collection<String> getReservedWords() {
        final Set<String> ret = new HashSet<>(super.getReservedWords());
        ret.addAll(reservedWords);
    	return ret;
    }

    @Override
    public String getDescription() {
	return "C++ and H++";
    }

    @Override
    public String getLanguage() {
	return "cpp";
    }
}
