////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: DelegatingLanguageTokenizer.java 231 2008-06-04 15:09:57Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.Iterator;
import java.util.Set;
import uk.co.ruelgnoj.orion.tokenizers.LanguageToken;
import uk.co.ruelgnoj.orion.tokenizers.LanguageTokenizer;
import uk.co.ruelgnoj.orion.tokenizers.TokenType;

/**
 * A tokenizer that delegated most requests.
 * @author Steve Millington
 * @version $Id: DelegatingLanguageTokenizer.java 231 2008-06-04 15:09:57Z steve $
 */
public abstract class DelegatingLanguageTokenizer implements LanguageTokenizer {
    private final LanguageTokenizer delegate;

    /**
     * Constructor.
     * <p>This calls the <code>createDelegate()</code> method to determine the
     * delegate to use.
     */
    public DelegatingLanguageTokenizer() {
	delegate = createDelegate();
	if (delegate == null) {
	    throw new NullPointerException("delegate");
	}
    }

    /**
     * Create the tokenizer to delegate to.
     * <p>This is called by the constructor.
     * @return The delegate
     */
    protected abstract LanguageTokenizer createDelegate();

    @Override
    public void setBuffer(CharSequence buffer) {
	delegate.setBuffer(buffer);
    }

    @Override
    public CharSequence getBuffer() {
	return delegate.getBuffer();
    }

    @Override
    public LanguageToken next() {
	return delegate.next();
    }

    @Override
    public boolean hasNext() {
	return delegate.hasNext();
    }

    @Override
    public boolean canContainDBEntries() {
	return delegate.canContainDBEntries();
    }

    @Override
    public Set<TokenType> getPossibleTokenTypes() {
	return delegate.getPossibleTokenTypes();
    }

    @Override
    public void remove() {
	delegate.remove();
    }

    @Override
    public Iterator<LanguageToken> iterator() {
	return delegate.iterator();
    }
}
