////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: InstallshieldLanguageTokenizer.java 285 2011-07-19 12:37:42Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;

/**
 * A tokenizer for Installshield(tm) scripts.
 *
 * @author Steve Millington
 * @version $Id: InstallshieldLanguageTokenizer.java 285 2011-07-19 12:37:42Z steve $
 */
public class InstallshieldLanguageTokenizer extends CLanguageTokenizer {
    //
    // The reserved words is a static table, that is initialized by the static
    // initializer. It is returned to the superclass when it asks for them.
    //
    private static final Collection<String> RES_WORDS;
    //
    // Static initialization of the reserved words
    //
    private static final String[] WORDS = {
	"if",
	"else",
	"endif",
	"case",
	"while",
	"switch",
	"abort",
	"begin",
	"end",
	"function",
	"number",
	"string",
	"bool",
	"int",
	"then",
	"return",
	"list",
	"endwhile",
	"endswitch",
	"default",
	"exit",
	"declare",
	"prototype",
	"pointer",
	"program",
	"goto",
	"false",
	"true"
    };

    static {
    	RES_WORDS = new HashSet<>(Arrays.asList(WORDS));
    }


    /**
     * Constructor.
     * @see uk.co.ruelgnoj.orion.tokenizers.impl.CLanguageTokenizer
     */
    public InstallshieldLanguageTokenizer() {
    	super();
    }

    @Override
    protected Collection<String> getReservedWords() {
    	return RES_WORDS;
    }

    @Override
    protected boolean isReservedWord(String s) {
    	return super.isReservedWord(s.toLowerCase());
    }

    @Override
    public String getDescription() {
    	return "Installshield";
    }

    @Override
    public String getLanguage() {
    	return "is";
    }
}
