////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: JavaLanguageTokenizer.java 285 2011-07-19 12:37:42Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;

/**
 * A tokenizer for Java source programs.
 *
 * @author Steve Millington
 * @version $Id: JavaLanguageTokenizer.java 285 2011-07-19 12:37:42Z steve $
 */
public class JavaLanguageTokenizer extends ClikeLanguageTokenizer {
    //
    // The reserved words is a static table, that is initialized by the static
    // initializer. It is returned to the superclass when it asks for them.
    //
    private static final Collection<String> RES_WORDS;
    private static final String[] WORDS = {
	"abstract",
	"assert",
	"boolean",
	"break",
	"byte",
	"case",
	"catch",
	"char",
	"class",
	"const",
	"continue",
	"default",
	"do",
	"double",
	"else",
	"enum",
	"extends",
	"false",
	"final",
	"finally",
	"float",
	"for",
	"goto",
	"if",
	"implements",
	"import",
	"instanceof",
	"int",
	"interface",
	"long",
	"native",
	"new",
	"null",
	"package",
	"private",
	"protected",
	"public",
	"return",
	"short",
	"static",
	"strictfp",
	"super",
	"switch",
	"synchronized",
	"this",
	"throw",
	"throws",
	"transient",
	"true",
	"try",
	"void",
	"volatile",
	"while"
    };

    static {
    	RES_WORDS = new HashSet<>(Arrays.asList(WORDS));
    }


    /**
     * Constructor.
     * @see uk.co.ruelgnoj.orion.tokenizers.impl.CLanguageTokenizer
     */
    public JavaLanguageTokenizer() {
    }

    @Override
    protected Collection<String> getReservedWords() {
    	return RES_WORDS;
    }

    @Override
    public String getDescription() {
    	return "Java";
    }

    @Override
    public String getLanguage() {
    	return "java";
    }
}
