////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: JavaLanguageTokenizer.java 226 2008-05-11 15:10:59Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.ResourceBundle;

/**
 * A tokenizer for Javascript source programs.
 *
 * @author Steve Millington
 * @version $Id: JavaLanguageTokenizer.java 226 2008-05-11 15:10:59Z steve $
 */
public class JavascriptLanguageTokenizer extends ClikeLanguageTokenizer {
    private static final String RBNAME =
		"uk.co.ruelgnoj.orion.tokenizers.resources";
    //
    // The reserved words is a static table, that is initialized by the static
    // initializer. It is returned to the superclass when it asks for them.
    //
    private static final Collection<String> RES_WORDS;
    private static final String[] WORDS = {
        "abstract",
        "as",
        "boolean",
        "break",
        "byte",
        "case",
        "catch",
        "char",
        "class",
        "continue",
        "const",
        "debugger",
        "default",
        "delete",
        "do",
        "double",
        "else",
        "enum",
        "export",
        "extends",
        "false",
        "final",
        "finally",
        "float",
        "for",
        "function",
        "goto",
        "if",
        "implements",
        "import",
        "in",
        "instanceof",
        "int",
        "interface",
        "is",
        "long",
        "namespace",
        "native",
        "new",
        "null",
        "package",
        "private",
        "protected",
        "public",
        "return",
        "short",
        "static",
        "super",
        "switch",
        "synchronized",
        "this",
        "throw",
        "throws",
        "transient",
        "true",
        "try",
        "typeof",
        "use",
        "var",
        "void",
        "volatile",
        "while",
        "with"
    };
    

    static {
        RES_WORDS = new HashSet<>(Arrays.asList(WORDS));
    }

    /**
     * Constructor.
     * @see uk.co.ruelgnoj.orion.tokenizers.impl.CLanguageTokenizer
     */
    public JavascriptLanguageTokenizer() {
    }

    @Override
    protected Collection<String> getReservedWords() {
        return RES_WORDS;
    }

    @Override
    public String getDescription() {
        return ResourceBundle.getBundle(RBNAME).getString("lang.javascript.desc");
    }

    @Override
    public String getLanguage() {
        return "javascript";
    }
}
