////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: UnixShellLanguageTokenizer.java 231 2008-06-04 15:09:57Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.HashSet;
import java.util.Set;
import uk.co.ruelgnoj.orion.tokenizers.LanguageToken;
import uk.co.ruelgnoj.orion.tokenizers.TokenType;
import uk.co.ruelgnoj.orion.tokenizers.TokenTypes;

/**
 * A tokenizer for UNIX shell scripts.
 *
 * @author Steve Millington
 * @version $Id: UnixShellLanguageTokenizer.java 231 2008-06-04 15:09:57Z steve $
 */
public class UnixShellLanguageTokenizer extends UnknownLanguageTokenizer {

    /**
     * Constructor.
     * @see uk.co.ruelgnoj.orion.tokenizers.impl.UnknownLanguageTokenizer
     */
    public UnixShellLanguageTokenizer() {
    }

    @Override
    public LanguageToken next() {
	final LanguageToken lt = super.next();
	if (lt.getType() == TokenTypes.TT_PLAINTEXT) {
	    final int tokLen = lt.getLength();
	    for (int i = 0; i < tokLen; i++) {
		if (buffer.charAt(lt.getStartOffset() + i) == '#') {
		    lt.setType(TokenTypes.TT_COMMENT);
		} else if (buffer.charAt(lt.getStartOffset() + i) != ' ') {
		    break;
		}
	    }
	}
	return lt;
    }

    @Override
    public String getDescription() {
    	return "Unix Shell Scripts";
    }

    @Override
    public String getLanguage() {
    	return "shell";
    }

    @Override
    public Set<TokenType> getPossibleTokenTypes() {
	final Set<TokenType> s = super.getPossibleTokenTypes();
	final Set<TokenType> ret = new HashSet<>(s);
	ret.add(TokenTypes.TT_COMMENT);
	return ret;
    }
}
