////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: UnknownLanguageTokenizer.java 231 2008-06-04 15:09:57Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.tokenizers.impl;

import java.util.HashSet;
import java.util.Set;
import uk.co.ruelgnoj.orion.tokenizers.LanguageToken;
import uk.co.ruelgnoj.orion.tokenizers.TokenType;
import uk.co.ruelgnoj.orion.tokenizers.TokenTypes;

/**
 * A tokenizer for files that have no other tokenizer.
 * <p>This just returns all lines as 21 tokens, one of plain text, and
 * the other as the newline token.
 */
public class UnknownLanguageTokenizer extends AbstractLanguageTokenizer {

    /** @see uk.co.ruelgnoj.orion.tokenizers.impl.AbstractLanguageTokenizer */
    public UnknownLanguageTokenizer() {
    }

    @Override
    public LanguageToken next() {
	//
	// Just return each line as a 'PLAINTEXT' and preserve newlines!
	//
	int sp = pos;

	if (buffer.charAt(sp) == '\n') {
	    pos++;
	    return createToken(sp, 1, TokenTypes.TT_NEWLINE);
	}
	int bLen = buffer.length();
    	while (pos < bLen) {
	    /*
	    if (buffer[pos] == '\r') {
		buffer[pos]=' ';
	    }
	    */
	    if (buffer.charAt(pos) == '\n') {
	    	break;
	    }
	    pos++;
	}
	return createToken(sp, pos - sp, TokenTypes.TT_PLAINTEXT);
    }

    @Override
    public String getDescription() {
    	return "Unknown";
    }

    @Override
    public String getLanguage() {
    	return "default";
    }

    @Override
    public Set<TokenType> getPossibleTokenTypes() {
	Set<TokenType> ret = new HashSet<>();
	ret.add(TokenTypes.TT_NEWLINE);
	ret.add(TokenTypes.TT_PLAINTEXT);
	return ret;
    }
}
