////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: AboutOrion.java 269 2008-07-22 06:52:09Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.application;

import uk.co.ruelgnoj.orion.product.Product;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Frame;
import java.awt.GridLayout;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import javax.swing.border.Border;

/**
 * The 'about' box for orion.
 *
 * @author Steve Millington
 * @version $Id: AboutOrion.java 269 2008-07-22 06:52:09Z steve $
 */
public class AboutOrion extends JDialog {

    private static final String RBNAME = "uk.co.ruelgnoj.orion.application.resources";
    private static final Border BORDER0055 = BorderFactory.createEmptyBorder(0, 0, 5, 5);
    private static final Border BORDER5555 = BorderFactory.createEmptyBorder(5, 5, 5, 5);

    /**
     * Constructor.
     *
     * @param f The frame to base the dialog on.
     */
    public AboutOrion(Frame f) {
        super(f);
        final ResourceBundle rb = ResourceBundle.getBundle(RBNAME, f.getLocale());

        setTitle(rb.getString("about.title"));
        setModal(true);
        getContentPane().setLayout(new BorderLayout());

        final JPanel p = new JPanel();
        p.setLayout(new GridLayout(0, 1));
        final JLabel l = new JLabel(
                rb.getString("about.message"), JLabel.CENTER);
        l.setForeground(Color.BLUE);
        p.add(l);
        p.add(new JLabel(Product.getBundle().getString("orion.version"), JLabel.CENTER));
        try {
            p.add(new JLabel(Product.getBundle().getString("orion.buildstamp"), JLabel.CENTER));
        } catch (MissingResourceException e) {
            // This is the case in a Netbeans build!
            p.add(new JLabel("Unknown", JLabel.CENTER));
        }
        p.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), BORDER5555));

        final JPanel p1 = new JPanel();
        p1.setLayout(new BorderLayout());

        p1.add(BorderLayout.CENTER, p);

        final JComponent c = new JLabel(
                new ImageIcon(getClass().getResource("/uk/co/ruelgnoj/orion/application/about_logo.jpg")));
        c.setBorder(BORDER0055);
        p1.add(BorderLayout.WEST, c);

        p1.add(BorderLayout.SOUTH, new JLabel(Product.getBundle().getString("orion.author"), JLabel.CENTER));

        p1.setBorder(BORDER0055);
        getContentPane().add(BorderLayout.CENTER, p1);

        pack();

        setLocationRelativeTo(f);
        setResizable(false);
    }
}
