////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: Orion.java 199 2008-01-14 17:04:44Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.application;

import java.net.MalformedURLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import uk.co.ruelgnoj.orion.project.Project;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;

import java.lang.reflect.Constructor;
import java.text.MessageFormat;
import java.util.HashMap;

import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import java.util.prefs.Preferences;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JEditorPane;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import uk.co.ruelgnoj.orion.project.ProjectFactory;
import uk.co.ruelgnoj.orion.product.Product;
import uk.co.ruelgnoj.orion.component.util.MnemonicHelper;
import uk.co.ruelgnoj.orion.component.util.RBAction;
import uk.co.ruelgnoj.orion.project.Category;

import uk.co.ruelgnoj.prefs.PreferenceManager;

/**
 * The Orion Application entry class.
 *
 * @author Steve Millington
 * @version $Id: Orion.java 199 2008-01-14 17:04:44Z steve $
 */
public class Orion extends JFrame {

    private static final String RBNAME = "uk.co.ruelgnoj.orion.application.resources";
    private static final String DEFAULT_OC_MANAGER = "uk.co.ruelgnoj.orion.application.TabbedPaneOCManager";
//       	"uk.co.ruelgnoj.orion.application.TreePanelOCManager";
    private OCManager ocManager;
    private final ProjectManager dbManager;

    /**
     * Constructor.
     *
     * @param title The title for the frame.
     * @param dbManager
     */
    public Orion(String title, ProjectManager dbManager) {
        super(title);
        this.dbManager = dbManager;

        PreferenceManager.getInstance().add(new LookAndFeelPI());

        //
        // Create a configuration for orion.
        //
        // First find the name of the properties file
        //
        /*
         Properties sysProps = System.getProperties();
         String userHome = sysProps.getProperty("user.home");
         String userName = sysProps.getProperty("user.name");
         cfgFile = new File(userHome, userName + ".orion.properties");
         props = new Properties();
         try {
         FileInputStream fis = new FileInputStream(cfgFile);
         BufferedInputStream bis = new BufferedInputStream(fis);
         props.load(fis);
         bis.close();
         fis.close();
         String s = sysProps.getProperty("orion.database");
         if (s != null) {
         props.put("orion.database", s);
         }
         } catch (IOException e) {
         String s = sysProps.getProperty("orion.database");
         if (s == null) {
         throw e;
         }
         props.put("orion.database", s);
         }
         */

 /*
         //
         // Note: Only 1 properties file for Orion Application
         //
         String dbName = props.getProperty("orion.database");
         if (dbName == null) {
         //
         // This is a bastardisation of the use of EOF Exception.
         // However, I can't be bothered to create a new Exception type!
         //
         throw new EOFException("orion.database");
         }
         */
        createOCManager();

        final List<Project> dd = dbManager.getInitialProjects();

        if (dd != null) {
            dd.stream().forEach((a) -> ocManager.add(a));
        }

        addMenuItems();

        getContentPane().setLayout(new BorderLayout());

        getContentPane().add(BorderLayout.CENTER, ocManager.getRootComponent());
    }

    /**
     * Create the oc manager.
     */
    private void createOCManager() {
        try {
            final Preferences prefs = Preferences.userNodeForPackage(getClass());
            final String ocManagerClassName = prefs.get("ocmanager.class", DEFAULT_OC_MANAGER);
            final Class<?> clazz = Class.forName(ocManagerClassName);
            final Constructor<?> constructor = clazz.getConstructor(Map.class);
            ocManager = (OCManager) constructor.newInstance(new HashMap<>());
        } catch (Exception e) {
            throw new Error(e);
        }
    }

    /**
     * Add the menu items.
     */
    private void addMenuItems() {
        final JMenuBar mb = new JMenuBar();

        JMenu m;

        m = createJMenu("file");
        m.addMenuListener(new MenuListener() {

            @Override
            public void menuDeselected(MenuEvent e) {
            }

            @Override
            public void menuCanceled(MenuEvent e) {
            }

            @Override
            public void menuSelected(MenuEvent e) {
                final JMenu m = (JMenu) e.getSource();
                m.removeAll();
                if (dbManager.allowsArbitraryProjects()) {
                    m.add(new OpenLocalAction());
                    m.add(new OpenWebAction());
                    m.add(new OpenJDBCAction());
                    m.addSeparator();
                }
                final List<Project> history = dbManager.getHistory();
                if (!history.isEmpty()) {
                    for (Project db : history) {
                        m.add(new ChooseDatabaseAction(db));
                    }
                    m.addSeparator();
                }
                final List<Category> predefined = dbManager.getPredefined();
                if (!predefined.isEmpty()) {
                    for (Category category: predefined) {
                        final JMenu submenu = new JMenu(category.getDescription());
                        for (Project p: category.getProjects()) {
                            submenu.add(new ChooseDatabaseAction(p));
                        }
                        m.add(submenu);
                    }
                    
                    m.addSeparator();
                }
                m.add(new ExitAction());
            }
        });
        mb.add(m);

        m = createJMenu("settings");
        m.add(new ConfigureAction());
        mb.add(m);

        m = createJMenu("help");
        m.add(new AboutAction());
//	mb.setHelpMenu(m);
        mb.add(m);

        setJMenuBar(mb);
    }

    /**
     * Action to open a file chooser for choosing an orion DB.
     */
    private class OpenLocalAction extends RBAction {

        /**
         * Constructor.
         */
        public OpenLocalAction() {
            super(ResourceBundle.getBundle(RBNAME),
                    "actions",
                    "openlocal");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            final JFileChooser jfc = new JFileChooser();
            jfc.setCurrentDirectory(new File(System.getProperty("user.dir")));
            jfc.setFileFilter(new OrionDBFilter());
            final int returnVal = jfc.showOpenDialog(Orion.this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                try {
                    final List<Category> cats = ProjectFactory.getCategories(jfc.getSelectedFile().toURI().toURL().toString());
                    cats.stream().map((cat) -> cat.getProjects()).forEach((pl) -> (pl).forEach((p) -> ocManager.add(p)));
//                    dbManager.use(dd.get(0));
                } catch (MalformedURLException ex) {
                    Logger.getLogger(Orion.class.getName()).log(Level.SEVERE, "Internal Logic Error", ex);
                }
            }
        }
    }

    /**
     * Action to open a file chooser for choosing an orion DB.
     */
    private class OpenWebAction extends RBAction {

        /**
         * Constructor.
         */
        public OpenWebAction() {
            super(ResourceBundle.getBundle(RBNAME),
                    "actions",
                    "openweb");
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            final OpenWebDialog jfc = new OpenWebDialog(Orion.this);
            jfc.setLocationRelativeTo(Orion.this);
            jfc.setVisible(true);
            if (jfc.isAccepted()) {
                final String url = jfc.getURL();
                final List<Category> cats = ProjectFactory.getCategories(url);
                cats.stream().map((cat) -> cat.getProjects()).forEach((pl) -> (pl).forEach((p) -> ocManager.add(p)));
//                dbManager.use(dd.get(0));
            }
        }
    }

    /**
     * Action to open a dialog choosing a JDBC orion DB.
     */
    private class OpenJDBCAction extends RBAction {

        /**
         * Constructor.
         */
        public OpenJDBCAction() {
            super(ResourceBundle.getBundle(RBNAME),
                    "actions",
                    "openjdbc");
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            final OpenJDBCDialog jfc = new OpenJDBCDialog(Orion.this);
            jfc.setLocationRelativeTo(Orion.this);
            jfc.setVisible(true);
            if (jfc.isAccepted()) {
                final String url = jfc.getURL();
                final List<Category> cats = ProjectFactory.getCategories(url);
                cats.stream().map((cat) -> cat.getProjects()).forEach((pl) -> (pl).forEach((p) -> ocManager.add(p)));
//                dbManager.use(cats.get(0));
            }
        }
    }

    /**
     * Action to terminate the Orion application.
     */
    private class ExitAction extends RBAction {

        /**
         * Constructor.
         */
        public ExitAction() {
            super(ResourceBundle.getBundle(RBNAME),
                    "actions",
                    "exit");
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

    /**
     * Action to display the about box.
     */
    private class AboutAction extends RBAction {

        /**
         * Constructor.
         */
        public AboutAction() {
            super(ResourceBundle.getBundle(RBNAME),
                    "actions",
                    "about");
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            final AboutOrion ao = new AboutOrion(Orion.this);
            ao.setVisible(true);
        }
    }

    /**
     * Action that chooses on of the history list of databases.
     */
    private class ChooseDatabaseAction extends AbstractAction {

        private final Project _file;

        /**
         * Constructor.
         *
         * @param file The DB file to open.
         */
        public ChooseDatabaseAction(Project database) {
            final String nameBase = ResourceBundle.getBundle(RBNAME).getString("actions.openhistory.name");
            putValue(Action.NAME, MessageFormat.format(nameBase, database.getName()));
            final String descBase = ResourceBundle.getBundle(RBNAME).getString("actions.openhistory.description");
            putValue(Action.SHORT_DESCRIPTION, MessageFormat.format(descBase, database.getName()));
            putValue(Action.LONG_DESCRIPTION, MessageFormat.format(descBase, database.getName()));
            putValue(Action.SMALL_ICON, database.getIcon());
            _file = database;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            ocManager.add(_file);
            dbManager.use(_file);
        }
    }

    /**
     * Action to bring up the configuation dialog.
     */
    private class ConfigureAction extends RBAction {

        /**
         * Constructor.
         */
        public ConfigureAction() {
            super(ResourceBundle.getBundle(RBNAME),
                    "actions",
                    "configure");
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            /*
             OrionComponent orionComponent =
             (OrionComponent)(tabs.getSelectedComponent());
             OrionConfigurator conf = new OrionConfigurator(
             Orion.this,
             orionComponent);
             conf.setVisible(true);
             if (conf.isOK()) {
             Properties p = conf.getResult();
             setDisplayPropertiesForAllOrions(p);
             if (conf.isSave()) {
             try {
             props.putAll(p);
             FileOutputStream fos = new FileOutputStream(cfgFile);
             props.save(fos, "Orion Configuration");
             fos.close();
             } catch (IOException ex) {
             System.err.println(ex);
             }
             }
             pack();
             }
             */
            final PreferenceManager pm = PreferenceManager.getInstance();

            pm.displayDialog(Orion.this);
        }
        /**
         * Set the display properties for all of the current in-play orion
         * components.
         *
         * @param props The properties
         */
        /*
         private void setDisplayPropertiesForAllOrions(Properties props) {
         Component[] comps = tabs.getComponents();
         for (int i = 0; i < comps.length; i++) {
         OrionComponent orionComponent = (OrionComponent)comps[i];
         orionComponent.setDisplayProperties(props);
         }
         }
         */
    }

    /**
     * Create a named menu.
     * <p>
     * The menuName is used as the key into the resource bundle to look up the
     * information for the menu.</p>
     *
     * @param menuName The name of the menu.
     * @return The menu
     */
    private JMenu createJMenu(String menuName) {
        final ResourceBundle rb = ResourceBundle.getBundle(RBNAME);
        final JMenu jm = new JMenu(rb.getString("menu." + menuName + ".name"));
        MnemonicHelper.assignMnemonic(jm, rb, "menu." + menuName);
        return jm;
    }

    /**
     * Entry Point.
     *
     * @param args The usual arguments to a java application
     */
    public static void main(String[] args) {
        if (args.length > 0) {
            SwingUtilities.invokeLater(() -> a(args[0]));
        } else {
            SwingUtilities.invokeLater(Orion::createAndShowGUI);
        }
    }

    private static void a(String s) {

        try {
            final ResourceBundle bundle = Product.getBundle();
            final String title = bundle.getString("orion.productname") + " " + bundle.getString("orion.version");
            final JFrame f = new Orion(title, new WSProjectManager(s));
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

            f.pack();
            f.setLocationRelativeTo(null);
            f.setVisible(true);
            f.requestFocus();
        } catch (Throwable e) {
            final JFrame f = new JFrame("Orion Error");
            final JEditorPane jep = new JEditorPane();
            final JScrollPane jsp = new JScrollPane(jep);
            f.getContentPane().add(jsp, BorderLayout.CENTER);
            final StringWriter sw = new StringWriter();
            try (final PrintWriter pw = new PrintWriter(sw)) {
                e.printStackTrace(pw);
            }
            jep.setText(sw.toString());
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            f.pack();
            f.setVisible(true);
        }
    }

    /**
     * Create and display the GUI.
     */
    public static void createAndShowGUI() {

        try {
            final ResourceBundle bundle = Product.getBundle();
            final String title = bundle.getString("orion.productname") + " " + bundle.getString("orion.version");
            final JFrame f = new Orion(title, new BasicProjectManager());
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

            f.pack();
            f.setLocationRelativeTo(null);
            f.setVisible(true);
            f.requestFocus();
        } catch (Throwable e) {
            final JFrame f = new JFrame("Orion Error");
            final JEditorPane jep = new JEditorPane();
            final JScrollPane jsp = new JScrollPane(jep);
            f.getContentPane().add(jsp, BorderLayout.CENTER);
            final StringWriter sw = new StringWriter();
            try (final PrintWriter pw = new PrintWriter(sw)) {
                e.printStackTrace(pw);
            }
            jep.setText(sw.toString());
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            f.pack();
            f.setVisible(true);
        }
    }
}
