////////////////////////////////////////////////////////////////////////////////
// orion: Provides a set of source code browsing utilities.
// Copyright (C) 2003  Steve Millington
//
// $Id: TreePanelOCManager.java 199 2008-01-14 17:04:44Z steve $
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package uk.co.ruelgnoj.orion.application;

import uk.co.ruelgnoj.orion.project.Project;
import uk.co.ruelgnoj.prefs.treepanel.TreePanel;

import javax.swing.JComponent;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ContainerListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ComponentEvent;

import java.util.ArrayList;
import java.util.Map;
import java.util.Set;
import uk.co.ruelgnoj.orion.component.FileViewSettings;
import uk.co.ruelgnoj.orion.component.OrionComponent;
import uk.co.ruelgnoj.orion.dm.OrionDataModelException;
import uk.co.ruelgnoj.prefs.PreferenceManager;

public class TreePanelOCManager implements OCManager {

    private final TreePanel _treePanel;
    private final Map<String, String> _props;
    private boolean _first = true;

    public TreePanelOCManager(Map<String, String> props) {
        _props = props;
        _treePanel = new TreePanel(new ArrayList<>());
        new ViewManager();
    }

    @Override
    public JComponent getRootComponent() {
        return _treePanel;
    }

    @Override
    public void add(Project dbDef) {
        final OrionComponent orionComponent = new OrionComponent(_props);
        if (_first) {

            Set<FileViewSettings> set = orionComponent.getSettingsSet();
            PreferenceManager.getInstance().add(
                    new FileViewColorsPI(set));
            _first = false;
        }

        try {
            orionComponent.setModel(dbDef.createDataModel());
        } catch (OrionDataModelException e) {
            e.printStackTrace();
        }

        orionComponent.addComponentListener(new ComponentAdapter() {

            @Override
            public void componentResized(ComponentEvent e) {
                /*
                 //
                 // Make ourself a proper amount of pixels wide and
                 // high.
                 //
                 Dimension sz = orionComponent.getSize();
                 Dimension psz = orionComponent.getPreferredSize();
                 int dw = psz.width - sz.width;
                 int dh = psz.height - sz.height;
                 if ((dw != 0) || (dh != 0)) {
                 Dimension bb = getSize();
                 //
                 // I don't know if this invalidation is required,
                 // but it works!
                 //
                 orionComponent.invalidate();
                 Component cl[] = orionComponent.getComponents();
                 for (int i = 0; i < cl.length; i++) {
                 cl[i].invalidate();
                 }
                 pack();
                 }
                 */
            }
        });
//	jtp.addTab(dbFile.getName(), orionComponent);
        //------------------------------------------------------------------
        // Test code
        //
        //orionComponent.setBorder(
        //	BorderFactory.createMatteBorder(10,20,30,40,Color.red));
        //------------------------------------------------------------------
//	jtp.setSelectedComponent(orionComponent);
//	orionComponent.requestFocus();
    }

    /**
     * A class to manage the jtabbed pane. It uses that 'current view' as the
     * trigger form performing add/remove operations and even calling
     * <code>System.exit()</code> when the last view is closed.
     */
    private class ViewManager implements PropertyChangeListener, ContainerListener {

        public ViewManager() {
            _treePanel.addContainerListener(this);
        }

        @Override
        public void propertyChange(PropertyChangeEvent e) {
            // "view"
            if (e.getNewValue() == null) {
            //
                // When a view changes to null, then we need to remove the
                // tab from the pane.
                //
//		jtp.remove((JComponent)e.getSource());
            }
        }

        @Override
        public void componentAdded(ContainerEvent e) {
            //
            // When a tab is added, we want to know about any view that it
            // has so that we can remove the tab when it is done with.
            //
            Component comp = e.getChild();
            comp.addPropertyChangeListener("view", this);
        }

        @Override
        public void componentRemoved(ContainerEvent e) {
            Component comp = e.getChild();
            comp.removePropertyChangeListener(this);

        //
            // When the last tab is removed, end the application
            //
//	    if (jtp.getTabCount() == 0) {
//		System.exit(0);
//	    }
        }
    }
}
