package uk.co.ruelgnoj.orion.application;

import java.util.prefs.Preferences;
import java.util.List;
import java.io.IOException;
import java.util.Collections;
import java.util.Set;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

import uk.co.ruelgnoj.orion.project.Category;
import uk.co.ruelgnoj.orion.project.Project;
import uk.co.ruelgnoj.orion.project.ProjectFactory;
import uk.co.ruelgnoj.orion.project.spi.ProjectFactoryImpl;

public class DBManagerTest {

    private static final ProjectFactoryImpl MH = new MockProjectFactoryImpl();
    private Preferences prefs;

    @Before
    public void before() {
        prefs = Preferences.userRoot();
        prefs = prefs.node("/ut.orion.DBFileManager");
        ProjectFactory.add(MH);
    }

    @After
    public void after() throws Exception {
        prefs.removeNode();
        ProjectFactory.remove(MH);
    }

    private BasicProjectManager createDBManager() {
        return new BasicProjectManager(prefs);
    }

    @Test
    public void isEmpty() {
        BasicProjectManager dbfm = createDBManager();
        List<Project> f = dbfm.getInitialProjects();
        assertNull(f);
        assertTrue(dbfm.getHistory().isEmpty());
    }

    @Test
    public void getInitialDB() throws IOException {
        final Project db1 = new MockOrionDatabaseDefinition("mock:tidb");
        BasicProjectManager dbfm = createDBManager();
        dbfm.setInitialProject(db1);
        List<Project> ret = dbfm.getInitialProjects();
        assertEquals(1, ret.size());
        assertEquals(db1.getURL(), ret.get(0).getURL());

        dbfm = createDBManager();
        ret = dbfm.getInitialProjects();
        assertEquals(db1, ret.get(0));
    }

    @Test
    public void basicUse() throws IOException {
        final Project db1 = new MockOrionDatabaseDefinition("mock:fish");
        final Project db2 = new MockOrionDatabaseDefinition("mock:fish");
        final Project db3 = new MockOrionDatabaseDefinition("mock:hake");
        BasicProjectManager dbfm = createDBManager();
        dbfm.use(db1);
        List<Project> history = dbfm.getHistory();
        assertEquals(1, history.size());
        dbfm.use(db2);
        assertEquals(1, history.size());
        dbfm.use(db3);
        history = dbfm.getHistory();
        assertEquals(2, history.size());
        assertEquals(db3, history.get(0));
        dbfm = createDBManager();
        history = dbfm.getHistory();
        assertEquals(2, history.size());
    }

    private static class MockProjectFactoryImpl extends ProjectFactoryImpl {

        @Override
        public Set<String> getProtocols() {
            return Collections.singleton("mock");
        }

        @Override
        protected List<Category> createProjectsImpl(String url) {
            return Collections.singletonList(new Category() {
                @Override
                public String getDescription() {
                    throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
                }
                
                @Override
                public List<Project> getProjects() {
                    return Collections.singletonList(new MockOrionDatabaseDefinition(url));
                }
               
            });
        }
    }
}

