/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2003 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/

package prefs.demo;

import javax.swing.JDialog;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.BorderFactory;
import javax.swing.UIManager;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.util.Locale;
import java.util.Set;
import java.util.ResourceBundle;
import java.util.prefs.Preferences;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.Icon;

import java.util.ResourceBundle;
import uk.co.ruelgnoj.prefs.AbstractPreferenceItem;
import uk.co.ruelgnoj.prefs.PreferenceCommitMechanism;
import uk.co.ruelgnoj.prefs.treepanel.util.VerticalFlowLayout;
import uk.co.ruelgnoj.prefs.treepanel.util.OptionLayout;

public class LookAndFeelPI extends AbstractPreferenceItem {
    private static final String RBNAME = "prefs.demo.resources";
    private String safeName;
    private LAFPIComponent lafPIComponent;
    private Icon icon;

    public LookAndFeelPI() {
	UIManagerX.initialize();

	Preferences prefs = Preferences.userNodeForPackage(getClass());
	String name = prefs.get("look.and.feel", null);
	if (name != null) {
	    useLAF(name);
	}
	safeName = UIManager.getLookAndFeel().getClass().getName();
	icon = new LAFIcon();
    }

    public Icon getIcon() {
	return icon;
    }

    protected JComponent createComponent() {
	lafPIComponent = new LAFPIComponent();
	return lafPIComponent;
    }

    private static class LAFPIComponent extends JPanel {
	private JList lafsList;
	private DefaultListModel lafsListModel;

	public LAFPIComponent() {
	    super(new VerticalFlowLayout());
	    JPanel panel = new JPanel(new BorderLayout());
	    lafsListModel = new DefaultListModel();
	    lafsList = new JList(lafsListModel);
	    updateLafsList();
	    lafsList.setCellRenderer(new LAFRenderer(lafsList.getCellRenderer()));

	    lafsList.addListSelectionListener(new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent e) {
		    if (e.getValueIsAdjusting()) {
			return;
		    }
		    JList list = (JList)e.getSource();
		    Object o = list.getSelectedValue();
		    if (o != null) {
			final UIManager.LookAndFeelInfo lafInfo = (UIManager.LookAndFeelInfo)o;
			SwingUtilities.invokeLater(new Runnable() {
			    public void run() {
				useLAF(lafInfo);
			    }
			});
		    }
		}
	    });
	    JScrollPane jsp = new JScrollPane(lafsList);
	    panel.add(BorderLayout.CENTER, jsp);

	    JPanel addPanel = new JPanel(new VerticalFlowLayout());
	    JPanel lafSpecPanel = new JPanel(new OptionLayout());
	    addPanel.setBorder(BorderFactory.createTitledBorder(
			getBundleString("header.register")));
	    JTextField lafName = new JTextField();
	    JTextField className = new JTextField();
	    JTextField classPath = new JTextField();

	    lafSpecPanel.add(new JLabel("LAF Name"));
	    lafSpecPanel.add(lafName);
	    lafSpecPanel.add(new JLabel("Class Name"));
	    lafSpecPanel.add(className);
	    lafSpecPanel.add(new JLabel("Class Path"));
	    lafSpecPanel.add(classPath);
	    addPanel.add(lafSpecPanel);
	    addPanel.add(new JButton(new AddAction(lafName, className, classPath)));

	    add(panel);
	    add(addPanel);
	}

	private void updateLafsList() {
	    UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
	    lafsListModel.removeAllElements();
	    for (UIManager.LookAndFeelInfo laf: lafs) {
		lafsListModel.addElement(laf);
	    }

	    String currentLafClassName =
		UIManager.getLookAndFeel().getClass().getName();
	    for (UIManager.LookAndFeelInfo laf: lafs) {
		if (laf.getClassName().equals(currentLafClassName)) {
		    lafsList.setSelectedValue(laf, true);
		}
	    }
	}

	private class AddAction extends AbstractAction {
	    private JTextField nameTF;
	    private JTextField classTF;
	    private JTextField jarTF;
	    public AddAction(JTextField nameTF,
		    JTextField classTF,
		    JTextField jarTF)  {
		this.nameTF = nameTF;
		this.classTF = classTF;
		this.jarTF = jarTF;
		putValue(Action.NAME, "Add");
	    }

	    public void actionPerformed(ActionEvent e) {
		UIManager.LookAndFeelInfo lafInfo = new LookAndFeelInfoExtended(
			nameTF.getText(),
			classTF.getText(),
			jarTF.getText());
		UIManagerX.installLookAndFeel(lafInfo);
		updateLafsList();
	    }
	}

	public UIManager.LookAndFeelInfo getLookAndFeelInfo() {
	    return (UIManager.LookAndFeelInfo)lafsList.getSelectedValue();
	}

    }

    private static void useLAF(UIManager.LookAndFeelInfo lafInfo) {
	try {
	    UIManagerX.setLookAndFeel(lafInfo);
	    updateFrames();
	} catch (UnsupportedLookAndFeelException e) {
	    e.printStackTrace();
	} catch (ClassNotFoundException e) {
	    e.printStackTrace();
	} catch (InstantiationException e) {
	    e.printStackTrace();
	} catch (IllegalAccessException e) {
	    e.printStackTrace();
	}
    }

    private static void updateFrames() {
	Frame[] frames = Frame.getFrames();
	for (Frame frame: frames) {
	    if (frame instanceof JFrame) {
		SwingUtilities.updateComponentTreeUI((JFrame)frame);
	    }
	}
    }

    private static void useLAF(String className) {
	try {
	    UIManagerX.setLookAndFeel(className);
	    updateFrames();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public String getDisplayName(Locale locale) {
	return getBundleString("displayname", locale);
    }

    protected void rollbackChangesImpl() {
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		useLAF(safeName);
	    }
	});
    }

    protected void commitChangesImpl(Set<PreferenceCommitMechanism> mechanisms) {
	Preferences prefs = Preferences.userNodeForPackage(getClass());
	String name = lafPIComponent.getLookAndFeelInfo().getClassName();
	prefs.put("look.and.feel", name);
	safeName = name;
    }

    private static String getBundleString(String name, Locale locale) {
	return ResourceBundle.getBundle(RBNAME, locale).getString(
		"lafpi." + name);
    }

    private static String getBundleString(String name) {
	return ResourceBundle.getBundle(RBNAME).getString("lafpi." + name);
    }


    private static class LAFRenderer implements ListCellRenderer {
	private ListCellRenderer other;

	public LAFRenderer(ListCellRenderer other) {
	    this.other = other;
	}

	public Component getListCellRendererComponent(JList list,
		Object value,
		int index,
		boolean isSelected,
		boolean cellHasFocus) {
	    if (value instanceof UIManager.LookAndFeelInfo) {
		value = ((UIManager.LookAndFeelInfo)value).getName();
	    }
	    return other.getListCellRendererComponent(list,
		    value,
		    index,
		    isSelected,
		    cellHasFocus);
	}
    }

    private static class LAFIcon implements Icon {
	public LAFIcon() {
	}

	public int getIconWidth() {
	    return 15;
	}

	public int getIconHeight() {
	    return 15;
	}

	public void paintIcon(java.awt.Component comp,
		java.awt.Graphics g1d,
		int x, int y) {
	    java.awt.Graphics2D g = (java.awt.Graphics2D)g1d;
	    java.awt.Paint p = g.getPaint();
	    g.setPaint(java.awt.Color.yellow);
	    g.fillRect(x + 2, y + 2, getIconWidth() - 2 - 2, getIconHeight() - 2 - 2);
	    g.setPaint(java.awt.Color.red);
	    g.fillRect(x + 4, y + 4, getIconWidth() - 4 - 4, getIconHeight() - 4 - 4);
	    g.setPaint(p);
	}
    }
}
