package prefs.demo;

import uk.co.ruelgnoj.prefs.PreferenceManager;

import uk.co.ruelgnoj.prefs.treepanel.AbstractTreePanelItem;
import uk.co.ruelgnoj.prefs.treepanel.TreePanel;
import uk.co.ruelgnoj.prefs.treepanel.TreePanelItem;

import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JEditorPane;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;

import javax.swing.BorderFactory;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

public class Main {
    private static final String RBNAME = "prefs.demo.resources";
    public static void main(String[] args) {
	PreferenceManager pm = new PreferenceManager();
	pm.add(new LookAndFeelPI());
	pm.add(new LocalePI());
	pm.add(new LoggingPI());

	createDemoFrame(pm);
    }

    private static void createDemoFrame(PreferenceManager pm) {

	JFrame frame = new JFrame("Preferences Demo");
	JDesktopPane jdp = new JDesktopPane();
	JInternalFrame jif = new JInternalFrame("TreePanel Example");

	List<TreePanelItem> items = new ArrayList<TreePanelItem>();
	for (int i = 0; i < 4; i++) {
	    items.add(new BasicTPI("btpi" + i));
	}
	TreePanel tp = new TreePanel(items);
	tp.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

	JEditorPane words = new JEditorPane("text/html",
		"<h1>Welcome to the Preferences Dialog Demo</h1><p>Below is an instance of a <code>TreePanel</code> with some data. Please select the preferences dialog from the menu to see an example of the preferences stuff.</p>");
	words.setEditable(false);
	words.setOpaque(false);

	frame.getContentPane().add(BorderLayout.CENTER, jdp);

	jif.getContentPane().add(BorderLayout.NORTH, words);
	jif.getContentPane().add(BorderLayout.CENTER, tp);
	jdp.add(jif);
	jif.pack();
	jif.setVisible(true);

	JMenu fileMenu = new JMenu("File");
	fileMenu.add(new ExitAction());
	JMenu sampleMenu = new JMenu("Sample");
	JMenuBar mb = new JMenuBar();
	mb.add(fileMenu);
	mb.add(sampleMenu);
	sampleMenu.add(pm.getShowPreferencesAction());
	frame.setJMenuBar(mb);

	frame.setSize(900,550);
	frame.setLocationRelativeTo(null);
	frame.setVisible(true);
    }

    private static class BasicTPI extends AbstractTreePanelItem {
	private String name;
	public BasicTPI(String name) {
	    this.name = name;
	}

	public String getDisplayName(Locale locale) {
	    return getBundleString("displayname", locale);
	}

	protected JComponent createComponent() {
	    return new JLabel(getBundleString("text"));
	}

	private String getBundleString(String key) {
	    return ResourceBundle.getBundle(RBNAME).getString(name + "." + key);
	}
	private String getBundleString(String key, Locale locale) {
	    return ResourceBundle.getBundle(RBNAME, locale).getString(name + "." + key);
	}
    }

    private static class ExitAction extends AbstractAction {
	public ExitAction() {
	    putValue(Action.NAME, "Exit");
	}

	public void actionPerformed(ActionEvent e) {
	    System.exit(0);
	}
    }
}
