package prefs.demo;

import javax.swing.UIManager;
import javax.swing.LookAndFeel;
import javax.swing.UnsupportedLookAndFeelException;
import java.util.prefs.Preferences;
import java.util.StringTokenizer;

public final class UIManagerX {
    private UIManagerX() {
    }

    public static void initialize() {
	Preferences prefs = Preferences.userNodeForPackage(UIManagerX.class);
	String names = prefs.get("laf.names", "");
	StringTokenizer st = new StringTokenizer(names,",");
	while (st.hasMoreTokens()) {
	    String name = st.nextToken();
	    String className = prefs.get("laf." + name + ".class", null);
	    String classPath = prefs.get("laf." + name + ".classpath", null);
	    UIManager.LookAndFeelInfo lafInfo = new LookAndFeelInfoExtended(
		    name,
		    className,
		    classPath);
	    UIManager.installLookAndFeel(lafInfo);
	}
    }

    public static void setLookAndFeel(String lafClassName)
	throws UnsupportedLookAndFeelException,
			  ClassNotFoundException,
			  InstantiationException,
			  IllegalAccessException {

	UIManager.LookAndFeelInfo lafInfo = findLAFInfo(lafClassName);
	if (lafInfo != null) {
	    setLookAndFeel(lafInfo);
	} else {
	    UIManager.setLookAndFeel(lafClassName);
	}
    }

    private static UIManager.LookAndFeelInfo findLAFInfo(String className) {
	UIManager.LookAndFeelInfo[] lis = UIManager.getInstalledLookAndFeels();
	for (UIManager.LookAndFeelInfo lafInfo: lis) {
	    if (lafInfo.getClassName().equals(className)) {
		return lafInfo;
	    }
	}
	return null;
    }

    public static void setLookAndFeel(UIManager.LookAndFeelInfo lafInfo)
	throws UnsupportedLookAndFeelException,
			  ClassNotFoundException,
			  InstantiationException,
			  IllegalAccessException {
	if (lafInfo instanceof LookAndFeelInfoExtended) {
	    LookAndFeelInfoExtended lafInfoX = (LookAndFeelInfoExtended)lafInfo;
	    LookAndFeel laf = lafInfoX.loadLookAndFeel();
	    Thread.currentThread().setContextClassLoader(
		    laf.getClass().getClassLoader());
	    UIManager.getLookAndFeelDefaults().put("ClassLoader",
		    laf.getClass().getClassLoader());
	    UIManager.setLookAndFeel(laf);
	} else {
	    UIManager.setLookAndFeel(lafInfo.getClassName());
	}
    }

    public static void installLookAndFeel(UIManager.LookAndFeelInfo lafInfo) {
	UIManager.installLookAndFeel(lafInfo);

	if (lafInfo instanceof LookAndFeelInfoExtended) {
	    LookAndFeelInfoExtended lafInfoX = (LookAndFeelInfoExtended)lafInfo;
	    Preferences prefs = Preferences.userNodeForPackage(UIManagerX.class);
	    String names = prefs.get("laf.names", "");
	    if (!names.equals("")) {
		names += ',';
	    }
	    String name = lafInfoX.getName();
	    names += name;
	    prefs.put("laf." + name + ".class", lafInfoX.getClassName());
	    prefs.put("laf." + name + ".classpath", lafInfoX.getClassPath());
	    prefs.put("laf.names", names);
	}
    }
}
