/**
 * *******************************************************************
 */
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**
 * *******************************************************************
 */
package uk.co.ruelgnoj.prefs;

import java.util.Set;

import uk.co.ruelgnoj.prefs.treepanel.AbstractTreePanelItem;

/**
 * A basic preference item.
 * <p>
 * This provides commit and rollback which are percolated to the children. This
 * is obviously in addition to the functionality provided by the superclass.
 */
public abstract class AbstractPreferenceItem extends AbstractTreePanelItem<PreferenceItem>
        implements PreferenceItem {

    /**
     * Constructor.
     */
    protected AbstractPreferenceItem() {
    }

    /**
     * Percolates call to children. {@inheritDoc}
     */
    @Override
    public final void commitChanges(Set<PreferenceCommitMechanism> commitMechanisms) {
        commitChangesImpl(commitMechanisms);
        for (PreferenceItem pi : getChildren()) {
            pi.commitChanges(commitMechanisms);
        }
        compNeedsUpdating = true;
    }

    /**
     * Percolates call to children. {@inheritDoc}
     */
    @Override
    public final void rollbackChanges() {
        rollbackChangesImpl();
        for (PreferenceItem pi : getChildren()) {
            pi.rollbackChanges();
        }
        compNeedsUpdating = true;
    }

    /**
     * Commit the changes for this preference item only.
     * <p>
     * Called by <code>commitChanges</code>
     *
     * @param commitMechanisms The commit mechanisms
     * @see #commitChanges(java.util.Set)
     */
    protected void commitChangesImpl(Set<PreferenceCommitMechanism> commitMechanisms) {
    }

    /**
     * Rollback the changes for this preference item only.
     * <p>
     * Called by <code>rollbackChanges</code>
     *
     * @see #rollbackChanges()
     */
    protected void rollbackChangesImpl() {
    }
}
