/**
 * *******************************************************************
 */
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**
 * *******************************************************************
 */
package uk.co.ruelgnoj.prefs;

import uk.co.ruelgnoj.prefs.util.EnhancedJDialog;

import uk.co.ruelgnoj.prefs.treepanel.TreePanel;

import java.awt.BorderLayout;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.ResourceBundle;

/**
 * The dialog displaying and managing the preferences.
 */
class PreferenceDialog extends EnhancedJDialog {

    private static final String BUNDLE_NAME = "uk.co.ruelgnoj.prefs.resources";

    private final List<PreferenceItem> items;

    public PreferenceDialog(JFrame f, List<PreferenceItem> items) {
        super(f, "", true);
        final ResourceBundle bundle = ResourceBundle.getBundle(BUNDLE_NAME, f.getLocale());
        setTitle(bundle.getString("dialog.title"));

        this.items = items;

        final JPanel treePanel = new TreePanel(items);
        treePanel.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));

        getContentPane().add(BorderLayout.CENTER, treePanel);
        pack();
        setLocationRelativeTo(f);
    }

    @Override
    protected boolean dialogOK() {
        // Call all the preferences items and accept their changes.
        final Set<PreferenceCommitMechanism> preferenceCommitItems = new HashSet<>();

        for (PreferenceItem pi : items) {
            pi.commitChanges(preferenceCommitItems);
        }

        for (PreferenceCommitMechanism pcm : preferenceCommitItems) {
            pcm.commit();
        }
        return true;
    }

    @Override
    protected boolean dialogCancel() {
        // Call all the preferences items and roll back their changes.
        for (PreferenceItem pi : items) {
            pi.rollbackChanges();
        }
        return true;
    }
}
