/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/
package uk.co.ruelgnoj.prefs;

import java.util.HashSet;
import java.util.Set;

import java.awt.Component;
import javax.swing.SwingUtilities;

/**
 * An item of preferences.
 * <p>Implementations of this interface are managed by the
 * <code>PreferenceManager</code> class.
 */
public class UIUpdateHandler implements PreferenceCommitMechanism {
    private static final UIUpdateHandler INSTANCE = new UIUpdateHandler();

    private final Set<Component> components;

    public static UIUpdateHandler getInstance() {
	return INSTANCE;
    }

    public UIUpdateHandler() {
	components = new HashSet<>();
    }

    @Override
    public void commit() {
	for (Component comp: components) {
	    SwingUtilities.updateComponentTreeUI(comp);
	}
    }

    public void registerComponent(Component comp) {
	components.add(comp);
    }

    public void deregisterComponent(Component comp) {
	components.remove(comp);
    }
}
