/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/
package uk.co.ruelgnoj.prefs.treepanel;

import uk.co.ruelgnoj.prefs.treepanel.util.VerticalFlowLayout;

import java.util.Collections;
import java.util.List;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 * A basic tree panel item.
 * @param <T>
 */
public abstract class AbstractTreePanelItem<T extends TreePanelItem> implements TreePanelItem<T> {

    protected JComponent comp;
    protected boolean compNeedsUpdating;
    private boolean wantsWholeSpace;

    /**
     * Constructor.
     */
    protected AbstractTreePanelItem() {
	wantsWholeSpace = false;
    }

    /**
     * No children.
     * {@inheritDoc}
     */
    @Override
    public List<T> getChildren() {
	List<T> ret = Collections.emptyList();
	return ret;
    }

    /**
     * No Icon.
     * {@inheritDoc}
     */
    @Override
    public Icon getIcon() {
	return null;
    }

    /**
     * Percolates call to children.
     * and then updates the UI of the component (if there is one).
     * {@inheritDoc}
     */
    @Override
    public void updateUI() {
	for (TreePanelItem tpi: getChildren()) {
	    tpi.updateUI();
	}
	if (comp != null) {
	    SwingUtilities.updateComponentTreeUI(comp);
	}
    }

    @Override
    public JComponent getComponent() {
	if (comp == null) {
	    comp = createComponent();
	    if (!wantsWholeSpace) {
		JPanel wrapper = new JPanel(new VerticalFlowLayout());
		wrapper.add(comp);
		comp = wrapper;
	    }
	    compNeedsUpdating = true;
	}
	if (compNeedsUpdating) {
	    updateComponent(comp);
	    compNeedsUpdating = false;
	}
	return comp;
    }

    /**
     * Create the component to be shown.
     * @return The new component.
     */
    protected abstract JComponent createComponent();

    /**
     * Update the presentation of the component.
     * <p>Default implementation does nothing!
     * @param comp The component
     */
    protected void updateComponent(JComponent comp) {
    }

    public boolean isWantingWholeSpace() {
	return wantsWholeSpace;
    }

    public void setWantingWholeSpace(boolean b) {
	wantsWholeSpace = b;
    }
}
