/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/
package uk.co.ruelgnoj.prefs.treepanel;

import java.util.List;
import java.util.Locale;

import javax.swing.JComponent;
import javax.swing.Icon;

/**
 * An item of preferences.
 * <p>Implementations of this interface are managed by the
 * <code>PreferenceManager</code> class.
 * @param <T>
 */
public interface TreePanelItem<T extends TreePanelItem> {
    /**
     * Get the display name of this tree panel item.
     * <p>This should be internationalized text.
     * @param locale The locale of the text required.
     * @return The display name.
     */
    public String getDisplayName(Locale locale);

    /**
     * Get an Icon to display in the tree.
     * @return The Icon, or null if no icon required
     */
    public Icon getIcon();

    /**
     * Get a component to allow the user to configure the item defined
     * here.
     * @return A component that must not be null.
     */
    public JComponent getComponent();

    /**
     * Update the UI of any stored components.
     */
    public void updateUI();

    /**
     * Get the child tree items.
     * @return The children (must not be null, but can be empty).
     */
    public List<T> getChildren();
}
