/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/
package uk.co.ruelgnoj.prefs.treepanel;

import java.awt.LayoutManager2;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Container;
import java.awt.Insets;

class TreePanelLayout implements LayoutManager2 {
    private static final int DEFAULT_X_GAP = 6;
    private static final int DEFAULT_Y_GAP = 6;

    private int xGap;
    private int yGap;

    public TreePanelLayout() {
	this(DEFAULT_X_GAP, DEFAULT_Y_GAP);
    }

    public TreePanelLayout(int xGap, int yGap) {
	this.xGap = xGap;
	this.yGap = yGap;
    }

    @Override
    public float getLayoutAlignmentX(Container container) {
	return 0.5f;
    }

    @Override
    public float getLayoutAlignmentY(Container container) {
	return 0.5f;
    }

    @Override
    public void invalidateLayout(Container container) {
    }

    @Override
    public void addLayoutComponent(Component comp, Object o) {
    }

    @Override
    public void addLayoutComponent(String s, Component comp) {
    }

    @Override
    public void removeLayoutComponent(Component comp) {
    }

    @Override
    public Dimension preferredLayoutSize(Container container) {
	Component[] children = container.getComponents();
	Dimension d0 = children[0].getPreferredSize();
	Dimension d1 = children[1].getPreferredSize();
	Dimension d2 = children[2].getPreferredSize();
	Dimension d3 = children[3].getPreferredSize();
	int w = Math.max(d0.width, d2.width)
		+ Math.max(d1.width, d3.width)
		+ xGap;
	int h = Math.max(d0.height, d1.height)
		+ Math.max(d2.height, d3.height)
		+ yGap;
	Insets insets = container.getInsets();
	w += insets.left + insets.right;
	h += insets.top + insets.bottom;
	return new Dimension(w, h);
    }

    @Override
    public Dimension minimumLayoutSize(Container container) {
	Component[] children = container.getComponents();
	Dimension d0 = children[0].getMinimumSize();
	Dimension d1 = children[1].getMinimumSize();
	Dimension d2 = children[2].getMinimumSize();
	Dimension d3 = children[3].getMinimumSize();
	int w = Math.max(d0.width, d2.width)
		+ Math.max(d1.width, d3.width)
		+ xGap;
	int h = Math.max(d0.height, d1.height)
		+ Math.max(d2.height, d3.height)
		+ yGap;
	Insets insets = container.getInsets();
	w += insets.left + insets.right;
	h += insets.top + insets.bottom;
	return new Dimension(w, h);
    }

    @Override
    public Dimension maximumLayoutSize(Container container) {
	Component[] children = container.getComponents();
	Dimension d0 = children[0].getMaximumSize();
	Dimension d1 = children[1].getMaximumSize();
	Dimension d2 = children[2].getMaximumSize();
	Dimension d3 = children[3].getMaximumSize();
	int w = Math.max(d0.width, d2.width)
		+ Math.max(d1.width, d3.width)
		+ xGap;
	int h = Math.max(d0.height, d1.height)
		+ Math.max(d2.height, d3.height)
		+ yGap;
	Insets insets = container.getInsets();
	w += insets.left + insets.right;
	h += insets.top + insets.bottom;
	return new Dimension(w, h);
    }

    @Override
    public void layoutContainer(Container container) {
	Component[] children = container.getComponents();
	Insets insets = container.getInsets();
	Dimension size = container.getSize();
	Dimension d0 = children[0].getPreferredSize();
	Dimension d1 = children[1].getPreferredSize();
	Dimension d2 = children[2].getPreferredSize();
	Dimension d3 = children[3].getPreferredSize();
	int pc1w = Math.max(d0.width, d2.width);
	int pc2w = Math.max(d1.width, d3.width);
	int pr1h = Math.max(d0.height, d1.height);
	int pr2h = Math.max(d2.height, d3.height);
	int c1w;
	int r1h;
	if (pc1w + insets.left + insets.right <= size.width) {
	    c1w = pc1w;
	} else {
	    c1w = size.width - insets.left - insets.right;
	}
	if (pr1h + insets.top + insets.bottom <= size.height) {
	    r1h = pr1h;
	} else {
	    r1h = size.height - insets.top - insets.bottom;
	}

	int c2w = size.width - c1w - insets.left - insets.right - xGap;
	if (c2w < 0) {
	    c2w = 0;
	}
	int r2h = size.height - r1h - insets.top - insets.bottom - yGap;
	if (r2h < 0) {
	    r2h = 0;
	}

	children[0].setBounds(insets.left,
		insets.top,
		c1w,
		r1h);
	children[1].setBounds(insets.left + c1w + xGap,
		insets.top,
		c2w,
		r1h);
	children[2].setBounds(
		insets.left,
		insets.top + r1h + yGap,
		c1w,
		r2h);
	children[3].setBounds(
		insets.left + c1w + xGap,
		insets.top + r1h + yGap,
		c2w,
		r2h);
    }
}
