/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/
package uk.co.ruelgnoj.prefs.treepanel;

import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;

import java.util.List;

/**
 * The tree model for the tree.
 */
public class TreePanelTreeModel implements TreeModel {
    private List<? extends TreePanelItem> items;
    public TreePanelTreeModel(List<? extends TreePanelItem> items) {
	if (items == null) {
	    throw new NullPointerException("items");
	}
	this.items = items;
    }

    @Override
    public Object getRoot() {
	return items;
    }

    @Override
    public Object getChild(Object parent, int index) {
	if (parent instanceof TreePanelItem) {
	    TreePanelItem tpi = (TreePanelItem)parent;
	    List l = tpi.getChildren();
	    if (l.size() > index) {
		return l.get(index);
	    } else {
		return null;
	    }
	} else if (parent instanceof List) {
	    List l = (List)parent;
	    if (l.size() > index) {
		return l.get(index);
	    } else {
		return null;
	    }
	} else {
	    return null;
	}
    }

    @Override
    public int getChildCount(Object parent) {
	if (parent instanceof TreePanelItem) {
	    TreePanelItem tpi = (TreePanelItem)parent;
	    return tpi.getChildren().size();
	} else if (parent instanceof List) {
	    List l = (List)parent;
	    return l.size();
	} else {
	    return 0;
	}
    }

    @Override
    public boolean isLeaf(Object node) {
	if (node instanceof TreePanelItem) {
	    TreePanelItem tpi = (TreePanelItem)node;
	    return tpi.getChildren().isEmpty();
	} else if (node instanceof List) {
	    List l = (List)node;
	    return l.isEmpty();
	} else {
	    return true;
	}
    }

    @Override
    public void valueForPathChanged(TreePath path, Object node) {
    }

    @Override
    public int getIndexOfChild(Object parent, Object child) {
	if (parent instanceof TreePanelItem) {
	    TreePanelItem tpi = (TreePanelItem)parent;
	    return tpi.getChildren().indexOf(child);
	} else if (parent instanceof List) {
	    List l = (List)parent;
	    return l.indexOf(child);
	} else {
	    return -1;
	}
    }

    @Override
    public void addTreeModelListener(TreeModelListener l) {
	// Model never changes to no listening required.
    }

    @Override
    public void removeTreeModelListener(TreeModelListener l) {
	// Model never changes to no listening required.
    }
}
