package uk.co.ruelgnoj.prefs.treepanel.util;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;

public class OptionLayout implements LayoutManager {

    private int hGap;
    private int vGap;
    private int hBorder;
    private int vBorder;

    public OptionLayout() {
        this(5, 5);
    }

    public OptionLayout(int i, int j) {
        hGap = i;
        vGap = j;
        hBorder = 0;
        vBorder = 0;
    }

    @Override
    public void addLayoutComponent(String s, Component component) {
    }

    @Override
    public void removeLayoutComponent(Component component) {
    }

    private static int getPreferredCol1Width(Component[] comps) {
        int i = 0;
	boolean include = true;
	for (Component comp : comps) {
	    if (include) {
		i = Math.max(i, comp.getPreferredSize().width);
	    }
	    include = !include;
	}

        return i;
    }

    private static int getPreferredCol2Width(Component[] comps) {
        int i = 0;
	boolean include = false;
	for (Component comp : comps) {
	    if (include) {
		i = Math.max(i, comp.getPreferredSize().width);
	    }
	    include = !include;
	}

        return i;
    }

    private static int getPreferredItemHeight(Component[] comps) {
        int i = 0;
	for (Component comp : comps) {
            i = Math.max(i, comp.getPreferredSize().height);
	}

        return i;
    }

    @Override
    public void layoutContainer(Container container) {
        final Insets insets = container.getInsets();
        final Component[] comps = container.getComponents();
        final int c1w = getPreferredCol1Width(comps);
        final int c2w = container.getSize().width - insets.left - insets.right - c1w - hGap - 2 * hBorder;
        final int k = getPreferredItemHeight(comps);
        final int l = (comps.length + 1) / 2;
        int h = k;
        final int j1 = container.getSize().height - insets.top - insets.bottom;
        if (k * l + vGap * (l - 1) < j1) {
            h = (j1 - vGap * (l - 1) - 2 * vBorder) / l;
	}
        int y = insets.top + vBorder;
        for (int i = 0; i < comps.length; i += 2) {
            final int c1x = hBorder + insets.left;
            final int c2x = c1x + c1w + hGap;
	    final Component comp1 = comps[i];
	    final Component comp2 = comps[i + 1];
	    final int hhh = Math.max(comp1.getPreferredSize().height,
		    comp2.getPreferredSize().height);
            comps[i].setBounds(c1x, y, c1w, hhh);
            if (comps.length > i + 1) {
                comps[i + 1].setBounds(c2x, y, c2w, hhh);
	    }
            y += hhh + vGap;
        }
    }

    @Override
    public Dimension minimumLayoutSize(Container container) {
        return new Dimension(0, 0);
    }

    @Override
    public Dimension preferredLayoutSize(Container container) {
        final Component[] comps = container.getComponents();
        final int c1w = getPreferredCol1Width(comps);
        final int c2w = getPreferredCol2Width(comps);
        final int ph = getPreferredItemHeight(comps);
        final int numRows = (comps.length + 1) / 2;
        final Insets insets = container.getInsets();
        final int wid = insets.left + insets.right + c1w + c2w + hGap + (2 * hBorder);
        int hig = insets.top + insets.bottom
	    + (vGap * (numRows - 1)) + (2 * vBorder);
	for (int i = 0; i < comps.length; i += 2) {
	    final Component comp1 = comps[i];
	    final Component comp2 = comps[i + 1];
	    final int h = Math.max(comp1.getPreferredSize().height,
		    comp2.getPreferredSize().height);
	    hig += h;
	}
        final Dimension dimension = new Dimension(wid, hig);
        return dimension;
    }
}
