package uk.co.ruelgnoj.prefs.treepanel.util;

import java.awt.LayoutManager2;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;

/**
 * A layout that makes all children completely fill the area provided.
 * It also sets the preferred size to be the outer bound of the preferred
 * sizes of the children.
 */
public class SingleLayout implements LayoutManager2 {
    /**
     * Constructor.
     */
    public SingleLayout() {
    }

    @Override
    public void invalidateLayout(Container c) {
    }

    @Override
    public float getLayoutAlignmentY(Container c) {
	return 0.0f;
    }

    @Override
    public float getLayoutAlignmentX(Container c) {
	return 0.0f;
    }

    @Override
    public Dimension maximumLayoutSize(Container c) {
	int w = Integer.MAX_VALUE;
	int h = Integer.MAX_VALUE;
	final Component[] children = c.getComponents();
	for (Component child: children) {
	    final Dimension d = child.getMaximumSize();
	    w = Math.min(w, d.width);
	    h = Math.min(h, d.height);
	}
	return new Dimension(w, h);
    }

    @Override
    public Dimension minimumLayoutSize(Container c) {
	int w = 0;
	int h = 0;
	final Component[] children = c.getComponents();
	for (Component child: children) {
	    final Dimension d = child.getMinimumSize();
	    w = Math.max(w, d.width);
	    h = Math.max(h, d.height);
	}
	return new Dimension(w, h);
    }

    @Override
    public Dimension preferredLayoutSize(Container c) {
	int w = 0;
	int h = 0;
	final Component[] children = c.getComponents();
	for (Component child: children) {
	    Dimension d = child.getPreferredSize();
	    w = Math.max(w, d.width);
	    h = Math.max(h, d.height);
	}
	final Insets insets = c.getInsets();
	return new Dimension(w + insets.left + insets.right,
	       	h + insets.top + insets.bottom);
    }

    @Override
    public void addLayoutComponent(Component c, Object o) {
    }

    @Override
    public void addLayoutComponent(String s, Component c) {
    }

    @Override
    public void removeLayoutComponent(Component c) {
    }

    @Override
    public void layoutContainer(Container c)  {
	final Insets insets = c.getInsets();
	final Component[] children = c.getComponents();
	for (Component child: children) {
	    child.setSize(c.getSize());
	    child.setLocation(insets.left, insets.top);
	}
    }
}
