package uk.co.ruelgnoj.prefs.treepanel.util;

import java.awt.Dimension;
import java.awt.Rectangle;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.junit.Ignore;
import org.junit.Test;
import static org.junit.Assert.*;

public class OptionLayoutTest {
    
    @Test
    public void withNull() {
	OptionLayout l = new OptionLayout();
	try {
	    l.layoutContainer(null);
	} catch (NullPointerException e) {
	}
    }

    @Test
    public void simple() {
	OptionLayout l = new OptionLayout(10, 20);
	JComponent c1 = new JPanel();
	c1.setPreferredSize(new Dimension(100, 200));
	JComponent c2 = new JPanel();
	c2.setPreferredSize(new Dimension(400, 200));
	JComponent container = new JPanel();
	container.setLayout(l);
	container.add(c1);
	container.add(c2);
	assertEquals(510, container.getPreferredSize().width);
	assertEquals(200, container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(new Rectangle(0, 0, 100, 200), c1.getBounds());
	assertEquals(new Rectangle(110, 0, 400, 200), c2.getBounds());
    }

    @Test
    public void simpleWithBorder() {
	OptionLayout l = new OptionLayout(10, 20);
	JComponent c1 = new JPanel();
	JComponent c2 = new JPanel();
	c1.setPreferredSize(new Dimension(100, 200));
	c2.setPreferredSize(new Dimension(400, 200));
	JComponent container = new JPanel();
	container.setBorder(BorderFactory.createEmptyBorder(20, 30, 70, 110));
	container.setLayout(l);
	container.add(c1);
	container.add(c2);
	assertEquals(500 + 30 + 110 + 10, container.getPreferredSize().width);
	assertEquals(200 + 20 + 70, container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(new Rectangle(30, 20, 100, 200), c1.getBounds());
	assertEquals(new Rectangle(140, 20, 400, 200), c2.getBounds());
    }

    @Test
    public void multipleWithBorder() {
	OptionLayout l = new OptionLayout(10, 20);
	JComponent c1 = new JPanel();
	JComponent c2 = new JPanel();
	JComponent c3 = new JPanel();
	JComponent c4 = new JPanel();
	c1.setPreferredSize(new Dimension(100, 200));
	c2.setPreferredSize(new Dimension(400, 200));
	c3.setPreferredSize(new Dimension(150, 100));
	c4.setPreferredSize(new Dimension(110, 150));
	JComponent container = new JPanel();
	container.setBorder(BorderFactory.createEmptyBorder(20, 30, 70, 110));
	container.setLayout(l);
	container.add(c1);
	container.add(c2);
	container.add(c3);
	container.add(c4);
	assertEquals(150 + 400 + 10 + 30 + 110, container.getPreferredSize().width);
	assertEquals(200 + 150 + 20 + 20 + 70, container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(new Rectangle(30, 20, 150, 200), c1.getBounds());
	assertEquals(new Rectangle(150 + 10 + 30, 20, 400, 200), c2.getBounds());
	assertEquals(new Rectangle(30, 200 + 20 + 20, 150, 150), c3.getBounds());
	assertEquals(new Rectangle(150 + 10 + 30, 200 + 20 + 20, 400, 150), c4.getBounds());
    }

    @Ignore
    @Test
    public void simpleWithBorder1() {
	OptionLayout l = new OptionLayout();
	JComponent c = new JPanel();
	c.setPreferredSize(new Dimension(100, 200));
	JComponent container = new JPanel();
	container.setBorder(BorderFactory.createEmptyBorder(20, 30, 70, 110));
	container.setLayout(l);
	container.add(c);
	assertEquals(c.getPreferredSize().width + 30 + 110,
		container.getPreferredSize().width);
	assertEquals(c.getPreferredSize().height + 20 + 70,
		container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(100 + 30 + 110, c.getWidth());
	assertEquals(200 + 20 + 70, c.getHeight());
    }

    @Ignore
    @Test
    public void multipleWithBorder1() {
	OptionLayout l = new OptionLayout();
	JComponent c1 = new JPanel();
	JComponent c2 = new JPanel();
	c1.setPreferredSize(new Dimension(100, 200));
	c2.setPreferredSize(new Dimension(300, 50));
	JComponent container = new JPanel();
	container.setBorder(BorderFactory.createEmptyBorder(20, 30, 70, 110));
	container.setLayout(l);
	container.add(c1);
	container.add(c2);
	assertEquals(300 + 30 + 110, container.getPreferredSize().width);
	assertEquals(200 + 20 + 70, container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(300 + 30 + 110, c1.getWidth());
	assertEquals(200 + 20 + 70, c1.getHeight());
	assertEquals(300 + 30 + 110, c2.getWidth());
	assertEquals(200 + 20 + 70, c2.getHeight());
    }
}
