package uk.co.ruelgnoj.prefs.treepanel.util;

import java.awt.Dimension;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.junit.Test;
import static org.junit.Assert.*;


public class SingleLayoutTest  {
    
    @Test
    public void doNull() {
	SingleLayout l = new SingleLayout();
	try {
	    l.layoutContainer(null);
	} catch (NullPointerException e) {
	}
    }

    @Test
    public void simple() {
	SingleLayout l = new SingleLayout();
	JComponent c = new JPanel();
	c.setPreferredSize(new Dimension(100, 200));
	JComponent container = new JPanel();
	container.setLayout(l);
	container.add(c);
	assertEquals(c.getPreferredSize().width,
		container.getPreferredSize().width);
	assertEquals(c.getPreferredSize().height,
		container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(100, c.getWidth());
	assertEquals(200, c.getHeight());
    }

    @Test
    public void simpleWithBorder() {
	SingleLayout l = new SingleLayout();
	JComponent c = new JPanel();
	c.setPreferredSize(new Dimension(100, 200));
	JComponent container = new JPanel();
	container.setBorder(BorderFactory.createEmptyBorder(20, 30, 70, 110));
	container.setLayout(l);
	container.add(c);
	assertEquals(c.getPreferredSize().width + 30 + 110,
		container.getPreferredSize().width);
	assertEquals(c.getPreferredSize().height + 20 + 70,
		container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(100 + 30 + 110, c.getWidth());
	assertEquals(200 + 20 + 70, c.getHeight());
    }

    @Test
    public void multipleWithBorder() {
	SingleLayout l = new SingleLayout();
	JComponent c1 = new JPanel();
	JComponent c2 = new JPanel();
	c1.setPreferredSize(new Dimension(100, 200));
	c2.setPreferredSize(new Dimension(300, 50));
	JComponent container = new JPanel();
	container.setBorder(BorderFactory.createEmptyBorder(20, 30, 70, 110));
	container.setLayout(l);
	container.add(c1);
	container.add(c2);
	assertEquals(300 + 30 + 110, container.getPreferredSize().width);
	assertEquals(200 + 20 + 70, container.getPreferredSize().height);

	container.setSize(container.getPreferredSize());
	container.doLayout();

	assertEquals(300 + 30 + 110, c1.getWidth());
	assertEquals(200 + 20 + 70, c1.getHeight());
	assertEquals(300 + 30 + 110, c2.getWidth());
	assertEquals(200 + 20 + 70, c2.getHeight());
    }
}
